# Copyright (c) 2025 Airbyte, Inc., all rights reserved.
"""Message cache for storing Airbyte messages from connector executions.

This module provides a DuckDB-based message cache for persisting and querying
Airbyte messages produced during connector test runs.

Based on airbyte-ci implementation:
https://github.com/airbytehq/airbyte/tree/master/airbyte-ci/connectors/live-tests/src/live_tests/commons/backends
"""

from airbyte_ops_mcp.regression_tests.message_cache.duckdb_cache import (
    DuckDbMessageCache,
)

__all__ = [
    "DuckDbMessageCache",
]



#
# Copyright (c) 2023 Airbyte, Inc., all rights reserved.
#

"""Module declaring context related classes."""

from __future__ import annotations

from datetime import datetime
from pathlib import Path
from types import TracebackType
from typing import TYPE_CHECKING

import yaml  # type: ignore
from asyncer import asyncify
from consts import BUILD_PLATFORMS
from dagger import Directory, Platform
from github import PullRequest
from pipelines.airbyte_ci.connectors.reports import ConnectorReport
from pipelines.dagger.actions import secrets
from pipelines.helpers.connectors.modifed import ConnectorWithModifiedFiles
from pipelines.helpers.execution.run_steps import RunStepOptions
from pipelines.helpers.github import update_commit_status_check
from pipelines.helpers.slack import send_message_to_webhook
from pipelines.helpers.utils import METADATA_FILE_NAME
from pipelines.models.contexts.pipeline_context import PipelineContext
from pipelines.models.secrets import LocalDirectorySecretStore, Secret, SecretStore

if TYPE_CHECKING:
    from pathlib import Path as NativePath
    from typing import Dict, FrozenSet, List, Optional, Sequence


class ConnectorContext(PipelineContext):
    """The connector context is used to store configuration for a specific connector pipeline run."""

    DEFAULT_CONNECTOR_ACCEPTANCE_TEST_IMAGE = "airbyte/connector-acceptance-test:dev"

    def __init__(
        self,
        pipeline_name: str,
        connector: ConnectorWithModifiedFiles,
        is_local: bool,
        git_branch: str,
        git_revision: str,
        diffed_branch: str,
        git_repo_url: str,
        report_output_prefix: str,
        ci_report_bucket: Optional[str] = None,
        ci_gcp_credentials: Optional[Secret] = None,
        ci_git_user: Optional[str] = None,
        ci_github_access_token: Optional[Secret] = None,
        connector_acceptance_test_image: str = DEFAULT_CONNECTOR_ACCEPTANCE_TEST_IMAGE,
        gha_workflow_run_url: Optional[str] = None,
        dagger_logs_url: Optional[str] = None,
        pipeline_start_timestamp: Optional[int] = None,
        ci_context: Optional[str] = None,
        slack_webhook: Optional[str] = None,
        pull_request: Optional[PullRequest.PullRequest] = None,
        should_save_report: bool = True,
        code_tests_only: bool = False,
        use_local_cdk: bool = False,
        use_cdk_ref: Optional[str] = None,
        use_host_gradle_dist_tar: bool = False,
        enable_report_auto_open: bool = True,
        docker_hub_username: Optional[Secret] = None,
        docker_hub_password: Optional[Secret] = None,
        s3_build_cache_access_key_id: Optional[Secret] = None,
        s3_build_cache_secret_key: Optional[Secret] = None,
        genai_api_key: Optional[Secret] = None,
        dbdocs_token: Optional[Secret] = None,
        concurrent_cat: Optional[bool] = False,
        run_step_options: RunStepOptions = RunStepOptions(),
        targeted_platforms: Sequence[Platform] = BUILD_PLATFORMS,
        secret_stores: Dict[str, SecretStore] | None = None,
    ) -> None:
        """Initialize a connector context.

        Args:
            connector (Connector): The connector under test.
            is_local (bool): Whether the context is for a local run or a CI run.
            git_branch (str): The current git branch name.
            git_revision (str): The current git revision, commit hash.
            diffed_branch: str: The branch to compare the current branch against.
            git_repo_url: str: The URL of the git repository.
            report_output_prefix (str): The S3 key to upload the test report to.
            connector_acceptance_test_image (Optional[str], optional): The image to use to run connector acceptance tests. Defaults to DEFAULT_CONNECTOR_ACCEPTANCE_TEST_IMAGE.
            gha_workflow_run_url (Optional[str], optional): URL to the github action workflow run. Only valid for CI run. Defaults to None.
            dagger_logs_url (Optional[str], optional): URL to the dagger logs. Only valid for CI run. Defaults to None.
            pipeline_start_timestamp (Optional[int], optional): Timestamp at which the pipeline started. Defaults to None.
            ci_context (Optional[str], optional): Pull requests, workflow dispatch or nightly build. Defaults to None.
            slack_webhook (Optional[str], optional): The slack webhook to send messages to. Defaults to None.
            pull_request (PullRequest, optional): The pull request object if the pipeline was triggered by a pull request. Defaults to None.
            code_tests_only (bool, optional): Whether to ignore non-code tests like QA and metadata checks. Defaults to False.
            use_host_gradle_dist_tar (bool, optional): Used when developing java connectors with gradle. Defaults to False.
            enable_report_auto_open (bool, optional): Open HTML report in browser window. Defaults to True.
            docker_hub_username (Optional[Secret], optional): Docker Hub username to use to read registries. Defaults to None.
            docker_hub_password (Optional[Secret], optional): Docker Hub password to use to read registries. Defaults to None.
            s3_build_cache_access_key_id (Optional[Secret], optional): Gradle S3 Build Cache credentials. Defaults to None.
            s3_build_cache_secret_key (Optional[Secret], optional): Gradle S3 Build Cache credentials. Defaults to None.
            concurrent_cat (bool, optional): Whether to run the CAT tests in parallel. Defaults to False.
            targeted_platforms (Optional[Iterable[Platform]], optional): The platforms to build the connector image for. Defaults to BUILD_PLATFORMS.
        """

        self.pipeline_name = pipeline_name
        self.connector = connector
        self.connector_acceptance_test_image = connector_acceptance_test_image
        self._secrets_dir: Optional[Directory] = None
        self._updated_secrets_dir: Optional[Directory] = None
        self.cdk_version: Optional[str] = None
        self.should_save_report = should_save_report
        self.code_tests_only = code_tests_only
        self.use_local_cdk = use_local_cdk
        self.use_cdk_ref = use_cdk_ref
        self.use_host_gradle_dist_tar = use_host_gradle_dist_tar
        self.enable_report_auto_open = enable_report_auto_open
        self.docker_hub_username = docker_hub_username
        self.docker_hub_password = docker_hub_password
        self.s3_build_cache_access_key_id = s3_build_cache_access_key_id
        self.s3_build_cache_secret_key = s3_build_cache_secret_key
        self.genai_api_key = genai_api_key
        self.dbdocs_token = dbdocs_token
        self.concurrent_cat = concurrent_cat
        self.targeted_platforms = targeted_platforms
        super().__init__(
            pipeline_name=pipeline_name,
            is_local=is_local,
            git_branch=git_branch,
            git_revision=git_revision,
            diffed_branch=diffed_branch,
            git_repo_url=git_repo_url,
            report_output_prefix=report_output_prefix,
            gha_workflow_run_url=gha_workflow_run_url,
            dagger_logs_url=dagger_logs_url,
            pipeline_start_timestamp=pipeline_start_timestamp,
            ci_context=ci_context,
            slack_webhook=slack_webhook,
            pull_request=pull_request,
            ci_report_bucket=ci_report_bucket,
            ci_gcp_credentials=ci_gcp_credentials,
            ci_git_user=ci_git_user,
            ci_github_access_token=ci_github_access_token,
            run_step_options=run_step_options,
            enable_report_auto_open=enable_report_auto_open,
            secret_stores=secret_stores,
        )

    @property
    def modified_files(self) -> FrozenSet[NativePath]:
        return self.connector.modified_files

    @property
    def secrets_dir(self) -> Optional[Directory]:
        return self._secrets_dir

    @secrets_dir.setter
    def secrets_dir(self, secrets_dir: Directory) -> None:
        self._secrets_dir = secrets_dir

    @property
    def updated_secrets_dir(self) -> Optional[Directory]:
        return self._updated_secrets_dir

    @updated_secrets_dir.setter
    def updated_secrets_dir(self, updated_secrets_dir: Directory) -> None:
        self._updated_secrets_dir = updated_secrets_dir

    @property
    def connector_acceptance_test_source_dir(self) -> Directory:
        return self.get_repo_dir("airbyte-integrations/bases/connector-acceptance-test")

    @property
    def live_tests_dir(self) -> Directory:
        return self.get_repo_dir("airbyte-ci/connectors/live-tests")

    @property
    def erd_package_dir(self) -> Directory:
        return self.get_repo_dir("airbyte-ci/connectors/erd")

    @property
    def should_save_updated_secrets(self) -> bool:
        return (
            self.ci_gcp_credentials is not None and self.updated_secrets_dir is not None
        )

    @property
    def host_image_export_dir_path(self) -> str:
        return "." if self.is_ci else "/tmp"

    @property
    def metadata_path(self) -> Path:
        return self.connector.code_directory / METADATA_FILE_NAME

    @property
    def metadata(self) -> dict:
        return yaml.safe_load(self.metadata_path.read_text())["data"]

    @property
    def docker_repository(self) -> str:
        return self.metadata["dockerRepository"]

    @property
    def docker_image_tag(self) -> str:
        return self.metadata["dockerImageTag"]

    @property
    def docker_image(self) -> str:
        return f"{self.docker_repository}:{self.docker_image_tag}"

    @property
    def local_secret_store_name(self) -> str:
        return f"{self.connector.technical_name}-local"

    @property
    def local_secret_store(self) -> Optional[LocalDirectorySecretStore]:
        connector_secrets_path = self.connector.code_directory / "secrets"
        if connector_secrets_path.is_dir():
            return LocalDirectorySecretStore(connector_secrets_path)
        return None

    async def get_connector_dir(
        self, exclude: Optional[List[str]] = None, include: Optional[List[str]] = None
    ) -> Directory:
        """Get the connector under test source code directory.

        Args:
            exclude ([List[str], optional): List of files or directories to exclude from the directory. Defaults to None.
            include ([List[str], optional): List of files or directories to include in the directory. Defaults to None.

        Returns:
            Directory: The connector under test source code directory.
        """
        vanilla_connector_dir = self.get_repo_dir(
            str(self.connector.code_directory), exclude=exclude, include=include
        )
        return await vanilla_connector_dir.with_timestamps(1)

    async def __aexit__(
        self,
        exception_type: Optional[type[BaseException]],
        exception_value: Optional[BaseException],
        traceback: Optional[TracebackType],
    ) -> bool:
        """Perform teardown operation for the ConnectorContext.

        On the context exit the following operations will happen:
            - Upload updated connector secrets back to Google Secret Manager
            - Write a test report in JSON format locally and to S3 if running in a CI environment
            - Update the commit status check on GitHub if running in a CI environment.
        It should gracefully handle the execution error that happens and always upload a test report and update commit status check.
        Args:
            exception_type (Optional[type[BaseException]]): The exception type if an exception was raised in the context execution, None otherwise.
            exception_value (Optional[BaseException]): The exception value if an exception was raised in the context execution, None otherwise.
            traceback (Optional[TracebackType]): The traceback if an exception was raised in the context execution, None otherwise.
        Returns:
            bool: Whether the teardown operation ran successfully.
        """
        self.stopped_at = datetime.utcnow()
        self.state = self.determine_final_state(self.report, exception_value)
        if exception_value:
            self.logger.error(
                "An error got handled by the ConnectorContext", exc_info=True
            )
        if self.report is None:
            self.logger.error(
                "No test report was provided. This is probably due to an upstream error"
            )
            self.report = ConnectorReport(self, [])

        if self.should_save_updated_secrets:
            await secrets.upload(self)

        self.report.print()

        if self.should_save_report:
            await self.report.save()

        await asyncify(update_commit_status_check)(**self.github_commit_status)

        if self.should_send_slack_message:
            # Using a type ignore here because the should_send_slack_message property is checking for non nullity of the slack_webhook
            await asyncify(send_message_to_webhook)(
                self.create_slack_message(),
                self.get_slack_channels(),
                self.slack_webhook,
            )  # type: ignore

        # Supress the exception if any
        return True



#
# Copyright (c) 2023 Airbyte, Inc., all rights reserved.
#

from typing import Any, Type


class Singleton:
    """
    A base class for implementing the Singleton pattern.

    This class stores instances and initialization flags for each subclass in dictionaries.
    This allows each subclass to have its own unique instance and control over its initialization process.

    The __new__ method ensures that only one instance of each subclass is created.
    The _initialized dictionary is used to control when the initialization logic of each subclass is executed.
    """

    _instances: dict[Type["Singleton"], Any] = {}
    _initialized: dict[Type["Singleton"], bool] = {}

    def __new__(cls: Type["Singleton"], *args: Any, **kwargs: Any) -> Any:
        if cls not in cls._instances:
            cls._instances[cls] = super().__new__(cls)
            cls._initialized[cls] = False
        return cls._instances[cls]



# Copyright (c) 2025 Airbyte, Inc., all rights reserved.
"""Airbyte Admin MCP server implementation.

This module provides the main MCP server for Airbyte admin operations.

The server can run in two modes:
- **stdio mode** (default): For direct MCP client connections via stdin/stdout
- **HTTP mode**: For HTTP-based MCP connections, useful for containerized deployments

Environment Variables:
    MCP_HTTP_HOST: Host to bind HTTP server to (default: 127.0.0.1)
    MCP_HTTP_PORT: Port for HTTP server (default: 8082)
"""

import asyncio
import os
import sys
from pathlib import Path

from airbyte.cloud.auth import resolve_cloud_client_id, resolve_cloud_client_secret
from dotenv import load_dotenv
from fastmcp import FastMCP
from fastmcp_extensions import MCPServerConfigArg, mcp_server

from airbyte_ops_mcp._sentry import init_sentry_tracking
from airbyte_ops_mcp.constants import (
    HEADER_AIRBYTE_CLOUD_CLIENT_ID,
    HEADER_AIRBYTE_CLOUD_CLIENT_SECRET,
    MCP_SERVER_NAME,
    ServerConfigKey,
)
from airbyte_ops_mcp.mcp._guidance import MCP_SERVER_INSTRUCTIONS
from airbyte_ops_mcp.mcp.agent_message_bus import register_message_bus_tools
from airbyte_ops_mcp.mcp.cloud_connector_versions import (
    register_cloud_connector_version_tools,
)
from airbyte_ops_mcp.mcp.connection_medic import register_connection_medic_tools
from airbyte_ops_mcp.mcp.connection_state import register_connection_state_tools
from airbyte_ops_mcp.mcp.connector_rollout import register_connector_rollout_tools
from airbyte_ops_mcp.mcp.devin_reminders import register_devin_reminder_tools
from airbyte_ops_mcp.mcp.devin_secret_request import register_devin_secret_request_tools
from airbyte_ops_mcp.mcp.gcp_logs import register_gcp_logs_tools
from airbyte_ops_mcp.mcp.github_actions import register_github_actions_tools
from airbyte_ops_mcp.mcp.github_repo_ops import register_github_repo_ops_tools
from airbyte_ops_mcp.mcp.human_in_the_loop import register_human_in_the_loop_tools
from airbyte_ops_mcp.mcp.people_lookup import register_people_lookup_tools
from airbyte_ops_mcp.mcp.prerelease import register_prerelease_tools
from airbyte_ops_mcp.mcp.prod_db_queries import register_prod_db_query_tools
from airbyte_ops_mcp.mcp.prompts import register_prompts
from airbyte_ops_mcp.mcp.registry import register_registry_tools
from airbyte_ops_mcp.mcp.regression_tests import register_regression_tests_tools
from airbyte_ops_mcp.mcp.session_feedback import register_session_feedback_tools
from airbyte_ops_mcp.mcp.session_namer import register_session_namer_tools
from airbyte_ops_mcp.mcp.slack_messaging import register_slack_messaging_tools
from airbyte_ops_mcp.mcp.tier_lookup import register_tier_lookup_tools

# Default HTTP server configuration
DEFAULT_HTTP_HOST = "127.0.0.1"
DEFAULT_HTTP_PORT = 8082


def _normalize_bearer_token(value: str) -> str | None:
    """Extract bearer token from Authorization header value.

    Parses "Bearer <token>" format (case-insensitive prefix).
    Returns None if the value doesn't have the Bearer prefix.
    """
    if value.lower().startswith("bearer "):
        token = value[7:].strip()
        return token if token else None
    return None


# Create the MCP server with built-in server info resource
app = mcp_server(
    name=MCP_SERVER_NAME,
    instructions=MCP_SERVER_INSTRUCTIONS,
    package_name="airbyte-internal-ops",
    advertised_properties={
        "docs_url": "https://github.com/airbytehq/airbyte-ops-mcp",
        "release_history_url": "https://github.com/airbytehq/airbyte-ops-mcp/releases",
    },
    server_config_args=[
        MCPServerConfigArg(
            name=ServerConfigKey.BEARER_TOKEN,
            http_header_key="Authorization",
            env_var="AIRBYTE_CLOUD_BEARER_TOKEN",
            normalize_fn=_normalize_bearer_token,
            required=False,
            sensitive=True,
        ),
        MCPServerConfigArg(
            name=ServerConfigKey.CLIENT_ID,
            http_header_key=HEADER_AIRBYTE_CLOUD_CLIENT_ID,
            default=lambda: str(resolve_cloud_client_id()),
            required=True,
            sensitive=True,
        ),
        MCPServerConfigArg(
            name=ServerConfigKey.CLIENT_SECRET,
            http_header_key=HEADER_AIRBYTE_CLOUD_CLIENT_SECRET,
            default=lambda: str(resolve_cloud_client_secret()),
            required=True,
            sensitive=True,
        ),
    ],
    include_standard_tool_filters=True,
)


def register_server_assets(app: FastMCP) -> None:
    """Register all server assets (tools, prompts, resources) with the FastMCP app.

    This function registers assets for all domains:
    - REPO: GitHub repository operations
    - CLOUD: Cloud connector version management
    - PROMPTS: Prompt templates for common workflows
    - REGRESSION_TESTS: Connector regression tests (single-version and comparison)
    - REGISTRY: Connector registry operations (read/write metadata from GCS)
    - METADATA: Connector metadata operations (future)
    - QA: Connector quality assurance (future)
    - INSIGHTS: Connector analysis and insights (future)

    Note: Server info resource is now built-in via mcp_server() helper.

    Args:
        app: FastMCP application instance
    """
    register_github_repo_ops_tools(app)
    register_github_actions_tools(app)
    register_prerelease_tools(app)
    register_cloud_connector_version_tools(app)
    register_connector_rollout_tools(app)
    register_prod_db_query_tools(app)
    register_gcp_logs_tools(app)
    register_prompts(app)
    register_regression_tests_tools(app)
    register_registry_tools(app)
    register_connection_state_tools(app)
    register_connection_medic_tools(app)
    register_people_lookup_tools(app)
    register_human_in_the_loop_tools(app)
    register_devin_reminder_tools(app)
    register_message_bus_tools(app)
    register_session_feedback_tools(app)
    register_session_namer_tools(app)
    register_slack_messaging_tools(app)
    register_devin_secret_request_tools(app)
    register_tier_lookup_tools(app)


register_server_assets(app)


def _load_env() -> None:
    """Load environment variables from .env file if present."""
    env_file = Path.cwd() / ".env"
    if env_file.exists():
        load_dotenv(env_file)
        print(f"Loaded environment from: {env_file}", flush=True, file=sys.stderr)


def main() -> None:
    """Main entry point for the Airbyte Admin MCP server (stdio mode).

    This is the default entry point that runs the server in stdio mode,
    suitable for direct MCP client connections.
    """
    _load_env()
    init_sentry_tracking()

    print("=" * 60, flush=True, file=sys.stderr)
    print("Starting Airbyte Admin MCP server (stdio mode).", file=sys.stderr)
    try:
        asyncio.run(app.run_stdio_async(show_banner=False))
    except KeyboardInterrupt:
        print("Airbyte Admin MCP server interrupted by user.", file=sys.stderr)

    print("Airbyte Admin MCP server stopped.", file=sys.stderr)
    print("=" * 60, flush=True, file=sys.stderr)


def _parse_port(port_str: str | None, default: int) -> int:
    """Parse and validate a port number from string.

    Args:
        port_str: Port string from environment variable, or None if not set
        default: Default port to use if port_str is None

    Returns:
        Validated port number

    Raises:
        ValueError: If port_str is not a valid integer or out of range
    """
    if port_str is None:
        return default

    port_str = port_str.strip()
    if not port_str.isdecimal():
        raise ValueError(f"MCP_HTTP_PORT must be a valid integer, got: {port_str!r}")

    port = int(port_str)
    if not 1 <= port <= 65535:
        raise ValueError(f"MCP_HTTP_PORT must be between 1 and 65535, got: {port}")

    return port


def main_http() -> None:
    """HTTP entry point for the Airbyte Admin MCP server.

    This entry point runs the server in HTTP mode, suitable for containerized
    deployments where the server needs to be accessible over HTTP.

    Environment Variables:
        MCP_HTTP_HOST: Host to bind to (default: 127.0.0.1)
        MCP_HTTP_PORT: Port to listen on (default: 8082)
    """
    _load_env()
    init_sentry_tracking()

    host = os.getenv("MCP_HTTP_HOST", DEFAULT_HTTP_HOST)
    port = _parse_port(os.getenv("MCP_HTTP_PORT"), DEFAULT_HTTP_PORT)

    print("=" * 60, flush=True, file=sys.stderr)
    print(
        f"Starting Airbyte Admin MCP server (HTTP mode) on {host}:{port}",
        file=sys.stderr,
    )
    try:
        app.run(transport="http", host=host, port=port)
    except KeyboardInterrupt:
        print("Airbyte Admin MCP server interrupted by user.", file=sys.stderr)

    print("Airbyte Admin MCP server stopped.", file=sys.stderr)
    print("=" * 60, flush=True, file=sys.stderr)


if __name__ == "__main__":
    main()



"""Unit tests for the session_namer module."""

import pytest

from airbyte_ops_mcp.session_namer import (
    NamingScheme,
    _load_seed_data,
    extract_session_id,
    generate_all_names,
    generate_friendly_name,
    get_namespace_size,
)


@pytest.mark.unit
@pytest.mark.parametrize(
    "identifier,scheme",
    [
        pytest.param("test-session-123", NamingScheme.SUPERHERO, id="superhero"),
        pytest.param("test-session-123", NamingScheme.SILLY_BUDDY, id="silly_buddy"),
        pytest.param("", NamingScheme.SUPERHERO, id="empty_string"),
        pytest.param(
            "b2a641e838214f91b50d0f88940ac119",
            NamingScheme.SUPERHERO,
            id="uuid_like",
        ),
    ],
)
def test_generate_friendly_name_deterministic(
    identifier: str, scheme: NamingScheme
) -> None:
    """Same identifier + scheme always produces the same Title Case two-word name."""
    name_a = generate_friendly_name(identifier, scheme)
    name_b = generate_friendly_name(identifier, scheme)
    assert name_a == name_b
    parts = name_a.split(" ")
    assert len(parts) == 2, f"Expected 2 words, got {len(parts)}: {name_a!r}"
    assert all(p.isalpha() for p in parts), f"Non-alpha tokens in: {name_a!r}"
    assert name_a == name_a.title(), f"Name not Title Case: {name_a!r}"


@pytest.mark.unit
@pytest.mark.parametrize(
    "id_a,id_b,scheme",
    [
        pytest.param(
            "session-alpha", "session-beta", NamingScheme.SUPERHERO, id="superhero"
        ),
        pytest.param(
            "session-alpha",
            "session-beta",
            NamingScheme.SILLY_BUDDY,
            id="silly_buddy",
        ),
    ],
)
def test_generate_friendly_name_different_ids(
    id_a: str, id_b: str, scheme: NamingScheme
) -> None:
    """Different identifiers produce different names (probabilistically)."""
    assert generate_friendly_name(id_a, scheme) != generate_friendly_name(id_b, scheme)


@pytest.mark.unit
def test_generate_all_names_matches_individual() -> None:
    """generate_all_names returns one entry per scheme, matching individual calls."""
    identifier = "consistency-check"
    all_names = generate_all_names(identifier)
    assert set(all_names.keys()) == {s.value for s in NamingScheme}
    for scheme in NamingScheme:
        assert all_names[scheme.value] == generate_friendly_name(identifier, scheme)


@pytest.mark.unit
@pytest.mark.parametrize(
    "scheme,min_size",
    [
        pytest.param(NamingScheme.SUPERHERO, 40_000, id="superhero"),
        pytest.param(NamingScheme.SILLY_BUDDY, 40_000, id="silly_buddy"),
    ],
)
def test_get_namespace_size(scheme: NamingScheme, min_size: int) -> None:
    """Each scheme has at least 40K combinations."""
    size = get_namespace_size(scheme)
    assert isinstance(size, int)
    assert size >= min_size


@pytest.mark.unit
def test_naming_scheme_members() -> None:
    """Enum has exactly the two expected members with correct string values."""
    assert NamingScheme.SUPERHERO.value == "superhero"
    assert NamingScheme.SILLY_BUDDY.value == "silly-buddy"
    assert len(NamingScheme) == 2


@pytest.mark.unit
@pytest.mark.parametrize(
    "identifier,scheme,expected_name",
    [
        pytest.param(
            "test-session-123",
            NamingScheme.SUPERHERO,
            "Teal Burglar",
            id="superhero_known",
        ),
        pytest.param(
            "test-session-123",
            NamingScheme.SILLY_BUDDY,
            "Sneaky Seymour",
            id="silly_buddy_known",
        ),
        pytest.param(
            "b2a641e838214f91b50d0f88940ac119",
            NamingScheme.SUPERHERO,
            "Hardy Paladin",
            id="superhero_uuid",
        ),
        pytest.param(
            "b2a641e838214f91b50d0f88940ac119",
            NamingScheme.SILLY_BUDDY,
            "Ratty Kate",
            id="silly_buddy_uuid",
        ),
    ],
)
def test_golden_vectors(
    identifier: str, scheme: NamingScheme, expected_name: str
) -> None:
    """Known (identifier, scheme) pairs always produce the exact expected name."""
    assert generate_friendly_name(identifier, scheme) == expected_name


@pytest.mark.unit
def test_yaml_seed_data_loads() -> None:
    """YAML seed file loads and contains expected keys with non-empty lists."""
    data = _load_seed_data()
    expected_keys = [
        "superhero_adjectives",
        "superhero_nouns",
        "silly_buddy_adjectives",
        "silly_buddy_names",
    ]
    for key in expected_keys:
        assert key in data, f"Missing key: {key}"
        assert isinstance(data[key], list), f"{key} is not a list"
        assert len(data[key]) > 100, f"{key} has too few entries: {len(data[key])}"


# ---------------------------------------------------------------------------
# extract_session_id tests
# ---------------------------------------------------------------------------

_KNOWN_ID = "b2a641e838214f91b50d0f88940ac119"


@pytest.mark.unit
@pytest.mark.parametrize(
    "raw_input,expected_id",
    [
        pytest.param(_KNOWN_ID, _KNOWN_ID, id="bare_id"),
        pytest.param(
            f"https://app.devin.ai/sessions/{_KNOWN_ID}",
            _KNOWN_ID,
            id="devin_url",
        ),
        pytest.param(
            f"https://app.devin.ai/sessions/{_KNOWN_ID}/",
            _KNOWN_ID,
            id="devin_url_trailing_slash",
        ),
        pytest.param(
            f"https://other.example.com/sessions/{_KNOWN_ID}",
            _KNOWN_ID,
            id="other_host_sessions_url",
        ),
        pytest.param(
            f"https://example.com/some/path/{_KNOWN_ID}",
            _KNOWN_ID,
            id="generic_url_last_segment",
        ),
        pytest.param("plain-string-no-url", "plain-string-no-url", id="plain_string"),
    ],
)
def test_extract_session_id(raw_input: str, expected_id: str) -> None:
    """extract_session_id returns the bare ID from various input formats."""
    assert extract_session_id(raw_input) == expected_id


@pytest.mark.unit
def test_url_and_bare_id_produce_same_name() -> None:
    """A session URL and its bare ID must produce identical names."""
    url = f"https://app.devin.ai/sessions/{_KNOWN_ID}"
    name_from_id = generate_friendly_name(_KNOWN_ID, NamingScheme.SILLY_BUDDY)
    name_from_url = generate_friendly_name(
        extract_session_id(url), NamingScheme.SILLY_BUDDY
    )
    assert name_from_id == name_from_url == "Ratty Kate"



#
# Copyright (c) 2023 Airbyte, Inc., all rights reserved.
#

from dagster import (
    DefaultSensorStatus,
    RunRequest,
    SensorDefinition,
    SensorEvaluationContext,
    SkipReason,
    build_resources,
    sensor,
)

from ..utils.dagster_helpers import string_array_to_hash


def registry_updated_sensor(job, resources_def) -> SensorDefinition:
    """
    This sensor is responsible for polling the registry folder in GCS for updates to our oss and cloud registries.
    If it has, it will trigger the given job.
    """

    @sensor(
        name=f"{job.name}_on_registry_updated",
        job=job,
        minimum_interval_seconds=(2 * 60),
        default_status=DefaultSensorStatus.STOPPED,
    )
    def registry_updated_sensor_definition(context: SensorEvaluationContext):
        context.log.info("Starting gcs_registry_updated_sensor")

        with build_resources(resources_def) as resources:
            context.log.info("Got resources for gcs_registry_updated_sensor")

            context.log.info(f"Old etag cursor: {context.cursor}")

            new_etags_cursor = string_array_to_hash(
                [
                    resources.latest_oss_registry_gcs_blob.etag,
                    resources.latest_cloud_registry_gcs_blob.etag,
                ]
            )

            context.log.info(f"New etag cursor: {new_etags_cursor}")

            # Note: ETAGs are GCS's way of providing a version number for a file
            # Another option would be to use the last modified date or MD5 hash
            if context.cursor == new_etags_cursor:
                context.log.info("No new registries in GCS bucket")
                return SkipReason("No new registries in GCS bucket")

            context.update_cursor(new_etags_cursor)
            context.log.info("New registries in GCS bucket")
            run_key = f"updated_registries:{new_etags_cursor}"
            return RunRequest(run_key=run_key)

    return registry_updated_sensor_definition



#
# Copyright (c) 2023 Airbyte, Inc., all rights reserved.
#

"""Module declaring context related classes."""

from __future__ import annotations

import logging
import os
from datetime import datetime
from functools import lru_cache
from glob import glob
from types import TracebackType
from typing import TYPE_CHECKING, Dict

from asyncer import asyncify
from consts import (
    MANUAL_PIPELINE_STATUS_CHECK_OVERRIDE_PREFIXES,
    CIContext,
    ContextState,
)
from dagger import Client, Directory, File, GitRepository, Service
from dagger import Secret as DaggerSecret
from github import PullRequest
from pipelines.airbyte_ci.connectors.reports import ConnectorReport
from pipelines.helpers.execution.run_steps import RunStepOptions
from pipelines.helpers.github import AIRBYTE_GITHUB_REPO_URL, update_commit_status_check
from pipelines.helpers.slack import send_message_to_webhook
from pipelines.helpers.utils import java_log_scrub_pattern
from pipelines.models.reports import Report
from pipelines.models.secrets import Secret, SecretStore

if TYPE_CHECKING:
    from typing import List, Optional


class PipelineContext:
    """The pipeline context is used to store configuration for a specific pipeline run."""

    _dagger_client: Optional[Client]
    _report: Optional[Report | ConnectorReport]
    dockerd_service: Optional[Service]
    started_at: Optional[datetime]
    stopped_at: Optional[datetime]

    secrets_to_mask: List[str]

    PRODUCTION = bool(
        os.environ.get("PRODUCTION", False)
    )  # Set this to True to enable production mode (e.g. to send PR comments)

    @lru_cache
    def get_default_excluded_files(self) -> list[str]:
        return (
            [".git", "airbyte-ci/connectors/pipelines/*"]
            + glob("**/build", recursive=True)
            + glob("**/.venv", recursive=True)
            + glob("**/secrets", recursive=True)
            + glob("**/__pycache__", recursive=True)
            + glob("**/*.egg-info", recursive=True)
            + glob("**/.vscode", recursive=True)
            + glob("**/.pytest_cache", recursive=True)
            + glob("**/.eggs", recursive=True)
            + glob("**/.mypy_cache", recursive=True)
            + glob("**/.DS_Store", recursive=True)
            + glob("**/airbyte_ci_logs", recursive=True)
            + glob("**/.gradle", recursive=True)
        )

    def __init__(
        self,
        pipeline_name: str,
        is_local: bool,
        git_branch: str,
        git_revision: str,
        diffed_branch: str,
        git_repo_url: str,
        report_output_prefix: str,
        gha_workflow_run_url: Optional[str] = None,
        dagger_logs_url: Optional[str] = None,
        pipeline_start_timestamp: Optional[int] = None,
        ci_context: Optional[str] = None,
        is_ci_optional: bool = False,
        slack_webhook: Optional[str] = None,
        pull_request: Optional[PullRequest.PullRequest] = None,
        ci_report_bucket: Optional[str] = None,
        ci_gcp_credentials: Optional[Secret] = None,
        ci_git_user: Optional[str] = None,
        ci_github_access_token: Optional[Secret] = None,
        run_step_options: RunStepOptions = RunStepOptions(),
        enable_report_auto_open: bool = True,
        secret_stores: Dict[str, SecretStore] | None = None,
    ) -> None:
        """Initialize a pipeline context.

        Args:
            pipeline_name (str): The pipeline name.
            is_local (bool): Whether the context is for a local run or a CI run.
            git_branch (str): The current git branch name.
            git_revision (str): The current git revision, commit hash.
            diffed_branch (str): The branch to diff against.
            git_repo_url (str): The git repository URL.
            report_output_prefix (str): The prefix to use for the report output.
            gha_workflow_run_url (Optional[str], optional): URL to the github action workflow run. Only valid for CI run. Defaults to None.
            dagger_logs_url (Optional[str], optional): URL to the dagger logs. Only valid for CI run. Defaults to None.
            pipeline_start_timestamp (Optional[int], optional): Timestamp at which the pipeline started. Defaults to None.
            ci_context (Optional[str], optional): Pull requests, workflow dispatch or nightly build. Defaults to None.
            is_ci_optional (bool, optional): Whether the CI is optional. Defaults to False.
            slack_webhook (Optional[str], optional): Slack webhook to send messages to. Defaults to None.
            pull_request (PullRequest, optional): The pull request object if the pipeline was triggered by a pull request. Defaults to None.
        """
        self.pipeline_name = pipeline_name
        self.is_local = is_local
        self.git_branch = git_branch
        self.git_revision = git_revision
        self.diffed_branch = diffed_branch
        self.git_repo_url = git_repo_url
        self.report_output_prefix = report_output_prefix
        self.gha_workflow_run_url = gha_workflow_run_url
        self.dagger_logs_url = dagger_logs_url
        self.pipeline_start_timestamp = pipeline_start_timestamp
        self.created_at = datetime.utcnow()
        self.ci_context = ci_context
        self.state = ContextState.INITIALIZED
        self.is_ci_optional = is_ci_optional
        self.slack_webhook = slack_webhook
        self.pull_request = pull_request
        self.logger = logging.getLogger(self.pipeline_name)
        self._dagger_client = None
        self._report = None
        self.dockerd_service = None
        self.ci_gcp_credentials = ci_gcp_credentials
        self.ci_report_bucket = ci_report_bucket
        self.ci_git_user = ci_git_user
        self.ci_github_access_token = ci_github_access_token
        self.started_at = None
        self.stopped_at = None
        self.secrets_to_mask = []
        self.run_step_options = run_step_options
        self.enable_report_auto_open = enable_report_auto_open
        self.secret_stores = secret_stores if secret_stores else {}
        update_commit_status_check(**self.github_commit_status)

    @property
    def dagger_client(self) -> Client:
        assert self._dagger_client is not None, (
            "The dagger client was not set on this PipelineContext"
        )
        return self._dagger_client

    @dagger_client.setter
    def dagger_client(self, dagger_client: Client) -> None:
        self._dagger_client = dagger_client

    @property
    def is_ci(self) -> bool:
        return self.is_local is False

    @property
    def is_pr(self) -> bool:
        return self.ci_context == CIContext.PULL_REQUEST

    @property
    def repo(self) -> GitRepository:
        return self.dagger_client.git(AIRBYTE_GITHUB_REPO_URL, keep_git_dir=True)

    @property
    def report(self) -> Report | ConnectorReport | None:
        return self._report

    @report.setter
    def report(self, report: Report | ConnectorReport) -> None:
        self._report = report

    @property
    def java_log_scrub_pattern_secret(self) -> Optional[DaggerSecret]:
        if not self.secrets_to_mask:
            return None
        return self.dagger_client.set_secret(
            "log_scrub_pattern", java_log_scrub_pattern(self.secrets_to_mask)
        )

    @property
    def github_commit_status(self) -> dict:
        """Build a dictionary used as kwargs to the update_commit_status_check function."""
        target_url: Optional[str] = self.gha_workflow_run_url

        if (
            self.remote_storage_enabled
            and self.state not in [ContextState.RUNNING, ContextState.INITIALIZED]
            and isinstance(self.report, ConnectorReport)
        ):
            target_url = self.report.html_report_url

        return {
            "sha": self.git_revision,
            "state": self.state.value["github_state"],
            "target_url": target_url,
            "description": self.state.value["description"],
            "context": self.pipeline_name,
            "should_send": self._should_send_status_check(),
            "logger": self.logger,
            "is_optional": self.is_ci_optional,
        }

    @property
    def should_send_slack_message(self) -> bool:
        return self.slack_webhook is not None

    @property
    def has_dagger_cloud_token(self) -> bool:
        return "_EXPERIMENTAL_DAGGER_CLOUD_TOKEN" in os.environ

    @property
    def dagger_cloud_url(self) -> Optional[str]:
        """Gets the link to the Dagger Cloud runs page for the current commit."""
        if self.is_local or not self.has_dagger_cloud_token:
            return None

        return f"https://alpha.dagger.cloud/changeByPipelines?filter=dagger.io/git.ref:{self.git_revision}"

    @property
    def remote_storage_enabled(self) -> bool:
        return bool(self.ci_report_bucket) and bool(self.ci_gcp_credentials)

    def _should_send_status_check(self) -> bool:
        should_send = self.is_pr or any(
            self.pipeline_name.startswith(override)
            for override in MANUAL_PIPELINE_STATUS_CHECK_OVERRIDE_PREFIXES
        )
        self.logger.info(f"Should send status check: {should_send}")
        return should_send

    def get_repo_file(self, file_path: str) -> File:
        """Get a file from the current repository.

        The file is extracted from the host file system.

        Args:
            file_path (str): Path to the file to get.

        Returns:
            Path: The selected repo file.
        """
        return self.dagger_client.host().file(file_path)

    def get_repo_dir(
        self,
        subdir: str = ".",
        exclude: Optional[List[str]] = None,
        include: Optional[List[str]] = None,
    ) -> Directory:
        """Get a directory from the current repository.

        The directory is extracted from the host file system.
        A couple of files or directories that could corrupt builds are exclude by default (check DEFAULT_EXCLUDED_FILES).

        Args:
            subdir (str, optional): Path to the subdirectory to get. Defaults to "." to get the full repository.
            exclude ([List[str], optional): List of files or directories to exclude from the directory. Defaults to None.
            include ([List[str], optional): List of files or directories to include in the directory. Defaults to None.

        Returns:
            Directory: The selected repo directory.
        """

        if exclude is None:
            exclude = self.get_default_excluded_files()
        else:
            exclude += self.get_default_excluded_files()
            exclude = list(set(exclude))
        exclude.sort()  # sort to make sure the order is always the same to not burst the cache. Casting exclude to set can change the order
        if subdir != ".":
            subdir = f"{subdir}/" if not subdir.endswith("/") else subdir
            exclude = [f.replace(subdir, "") for f in exclude if subdir in f]
        return self.dagger_client.host().directory(
            subdir, exclude=exclude, include=include
        )

    def create_slack_message(self) -> str:
        raise NotImplementedError()

    def get_slack_channels(self) -> List[str]:
        raise NotImplementedError()

    async def __aenter__(self) -> PipelineContext:
        """Perform setup operation for the PipelineContext.

        Updates the current commit status on Github.

        Raises:
            Exception: An error is raised when the context was not initialized with a Dagger client
        Returns:
            PipelineContext: A running instance of the PipelineContext.
        """
        if self.dagger_client is None:
            raise Exception(
                "A Pipeline can't be entered with an undefined dagger_client"
            )
        self.state = ContextState.RUNNING
        self.started_at = datetime.utcnow()
        self.logger.info("Caching the latest CDK version...")
        await asyncify(update_commit_status_check)(**self.github_commit_status)
        if self.should_send_slack_message:
            # Using a type ignore here because the should_send_slack_message property is checking for non nullity of the slack_webhook
            await asyncify(send_message_to_webhook)(
                self.create_slack_message(),
                self.get_slack_channels(),
                self.slack_webhook,
            )  # type: ignore
        return self

    @staticmethod
    def determine_final_state(
        report: Optional[Report], exception_value: Optional[BaseException]
    ) -> ContextState:
        """Determine the final state of the context from the report or the exception value.

        Args:
            report (Optional[Report]): The pipeline report if any.
            exception_value (Optional[BaseException]): The exception value if an exception was raised in the context execution, None otherwise.
        Returns:
            ContextState: The final state of the context.
        """
        if exception_value is not None or report is None:
            return ContextState.ERROR
        if report is not None and report.considered_failed_steps:
            return ContextState.FAILURE
        if report is not None and report.success:
            return ContextState.SUCCESSFUL
        raise Exception(
            f"The final state of the context could not be determined for the report and exception value provided. Report: {report}, Exception: {exception_value}"
        )

    async def __aexit__(
        self,
        exception_type: Optional[type[BaseException]],
        exception_value: Optional[BaseException],
        traceback: Optional[TracebackType],
    ) -> bool:
        """Perform teardown operation for the PipelineContext.

        On the context exit the following operations will happen:
            - Log the error value if an error was handled.
            - Log the test report.
            - Update the commit status check on GitHub if running in a CI environment.

        It should gracefully handle all the execution errors that happened and always upload a test report and update commit status check.

        Args:
            exception_type (Optional[type[BaseException]]): The exception type if an exception was raised in the context execution, None otherwise.
            exception_value (Optional[BaseException]): The exception value if an exception was raised in the context execution, None otherwise.
            traceback (Optional[TracebackType]): The traceback if an exception was raised in the context execution, None otherwise.
        Returns:
            bool: Whether the teardown operation ran successfully.
        """
        if exception_value:
            self.logger.error("An error was handled by the Pipeline", exc_info=True)

        if self.report is None:
            self.logger.error(
                "No test report was provided. This is probably due to an upstream error"
            )
            self.report = Report(self, steps_results=[])

        self.state = self.determine_final_state(self.report, exception_value)
        self.stopped_at = datetime.utcnow()

        self.report.print()

        await asyncify(update_commit_status_check)(**self.github_commit_status)
        if self.should_send_slack_message:
            # Using a type ignore here because the should_send_slack_message property is checking for non nullity of the slack_webhook
            await asyncify(send_message_to_webhook)(
                self.create_slack_message(),
                self.get_slack_channels(),
                self.slack_webhook,  # type: ignore
            )
        # supress the exception if it was handled
        return True



# Copyright (c) 2024 Airbyte, Inc., all rights reserved.

from orchestrator.assets.connector_metrics import _convert_json_to_metrics_dict


def test_convert_json_to_metrics_dict():
    jsonl_string = (
        '{"_airbyte_ab_id":"4043ed5a-2fea-4edd-8e42-5fafb5521bae","_airbyte_emitted_at":1716765524746,"_airbyte_data":{"connector_type":"destination","connector_definition_id":"0b75218b-f702-4a28-85ac-34d3d84c0fc2","connector_name":"Chroma","docker_repository":"airbyte/destination-chroma","airbyte_platform":"all","connector_version":"all","sync_success_rate":"low","usage":"medium"}}\n'
        '{"_airbyte_ab_id":"b99b6003-551a-4732-8ef5-dab626745977","_airbyte_emitted_at":1716765524746,"_airbyte_data":{"connector_type":"destination","connector_definition_id":"0b75218b-f702-4a28-85ac-34d3d84c0fc2","connector_name":"Chroma","docker_repository":"airbyte/destination-chroma","airbyte_platform":"oss","connector_version":"0.0.10","sync_success_rate":"low","usage":"medium"}}\n'
        '{"_airbyte_ab_id":"753037c6-80d8-408b-93b6-135b94e75361","_airbyte_emitted_at":1716765524746,"_airbyte_data":{"connector_type":"source","connector_definition_id":"28ce1fbd-1e15-453f-aa9f-da6c4d928e92","connector_name":"Vantage","docker_repository":"airbyte/source-vantage","airbyte_platform":"oss","connector_version":"0.1.0","sync_success_rate":"low","usage":"medium"}}\n'
        '{"_airbyte_ab_id":"a5d8989c-5944-48ea-b80b-8a69baef6f15","_airbyte_emitted_at":1716765524746,"_airbyte_data":{"connector_type":"source","connector_definition_id":"28ce1fbd-1e15-453f-aa9f-da6c4d928e92","connector_name":"Vantage","docker_repository":"airbyte/source-vantage","airbyte_platform":"cloud","connector_version":"0.1.0","sync_success_rate":"low","usage":"medium"}}\n'
        '{"_airbyte_ab_id":"4204f086-b8f9-4b74-94bb-13d418d4724d","_airbyte_emitted_at":1716765524746,"_airbyte_data":{"connector_type":"source","connector_definition_id":"28ce1fbd-1e15-453f-aa9f-da6c4d928e92","connector_name":"Vantage","docker_repository":"airbyte/source-vantage","airbyte_platform":"all","connector_version":"all","sync_success_rate":"low","usage":"medium"}}\n'
    )

    expected_metrics_dict = {
        "0b75218b-f702-4a28-85ac-34d3d84c0fc2": {
            "all": {
                "connector_type": "destination",
                "connector_definition_id": "0b75218b-f702-4a28-85ac-34d3d84c0fc2",
                "connector_name": "Chroma",
                "docker_repository": "airbyte/destination-chroma",
                "airbyte_platform": "all",
                "connector_version": "all",
                "sync_success_rate": "low",
                "usage": "medium",
            },
            "oss": {
                "connector_type": "destination",
                "connector_definition_id": "0b75218b-f702-4a28-85ac-34d3d84c0fc2",
                "connector_name": "Chroma",
                "docker_repository": "airbyte/destination-chroma",
                "airbyte_platform": "oss",
                "connector_version": "0.0.10",
                "sync_success_rate": "low",
                "usage": "medium",
            },
        },
        "28ce1fbd-1e15-453f-aa9f-da6c4d928e92": {
            "oss": {
                "connector_type": "source",
                "connector_definition_id": "28ce1fbd-1e15-453f-aa9f-da6c4d928e92",
                "connector_name": "Vantage",
                "docker_repository": "airbyte/source-vantage",
                "airbyte_platform": "oss",
                "connector_version": "0.1.0",
                "sync_success_rate": "low",
                "usage": "medium",
            },
            "cloud": {
                "connector_type": "source",
                "connector_definition_id": "28ce1fbd-1e15-453f-aa9f-da6c4d928e92",
                "connector_name": "Vantage",
                "docker_repository": "airbyte/source-vantage",
                "airbyte_platform": "cloud",
                "connector_version": "0.1.0",
                "sync_success_rate": "low",
                "usage": "medium",
            },
            "all": {
                "connector_type": "source",
                "connector_definition_id": "28ce1fbd-1e15-453f-aa9f-da6c4d928e92",
                "connector_name": "Vantage",
                "docker_repository": "airbyte/source-vantage",
                "airbyte_platform": "all",
                "connector_version": "all",
                "sync_success_rate": "low",
                "usage": "medium",
            },
        },
    }

    metrics_dict = _convert_json_to_metrics_dict(jsonl_string)

    assert metrics_dict == expected_metrics_dict



# Copyright (c) 2025 Airbyte, Inc., all rights reserved.
"""Catalog validation functions for discovered catalogs.

Based on airbyte-ci validation tests:
https://github.com/airbytehq/airbyte/blob/master/airbyte-ci/connectors/live-tests/src/live_tests/validation_tests/test_discover.py
"""

from __future__ import annotations

from dataclasses import dataclass, field
from typing import Any

import dpath.util
import jsonschema
from airbyte_protocol.models import AirbyteCatalog, AirbyteStream


@dataclass
class ValidationResult:
    """Result of a validation check."""

    passed: bool
    message: str
    errors: list[str] = field(default_factory=list)
    warnings: list[str] = field(default_factory=list)

    @classmethod
    def success(cls, message: str = "Validation passed") -> ValidationResult:
        return cls(passed=True, message=message)

    @classmethod
    def failure(cls, message: str, errors: list[str] | None = None) -> ValidationResult:
        return cls(passed=False, message=message, errors=errors or [])


def validate_catalog_has_streams(catalog: AirbyteCatalog) -> ValidationResult:
    """Validate that the catalog has at least one stream.

    Args:
        catalog: The discovered catalog to validate.

    Returns:
        ValidationResult indicating success or failure.
    """
    if not catalog.streams:
        return ValidationResult.failure(
            "Catalog should contain at least one stream",
            errors=["No streams found in catalog"],
        )
    return ValidationResult.success(f"Catalog contains {len(catalog.streams)} streams")


def validate_no_duplicate_stream_names(catalog: AirbyteCatalog) -> ValidationResult:
    """Validate that all stream names in the catalog are unique.

    Args:
        catalog: The discovered catalog to validate.

    Returns:
        ValidationResult indicating success or failure.
    """
    name_counts: dict[str, int] = {}
    for stream in catalog.streams:
        count = name_counts.get(stream.name, 0)
        name_counts[stream.name] = count + 1

    duplicates = [name for name, count in name_counts.items() if count > 1]
    if duplicates:
        return ValidationResult.failure(
            f"Catalog has duplicate stream names: {duplicates}",
            errors=[f"Stream '{name}' appears multiple times" for name in duplicates],
        )
    return ValidationResult.success("All stream names are unique")


def validate_schemas_are_valid_json_schema(catalog: AirbyteCatalog) -> ValidationResult:
    """Validate that all stream schemas are valid JSON Schema Draft 7.

    Args:
        catalog: The discovered catalog to validate.

    Returns:
        ValidationResult indicating success or failure.
    """
    errors = []
    for stream in catalog.streams:
        try:
            jsonschema.Draft7Validator.check_schema(stream.json_schema)
        except jsonschema.exceptions.SchemaError as e:
            errors.append(
                f"Stream '{stream.name}' has invalid JSON schema: {e.message}"
            )

    if errors:
        return ValidationResult.failure(
            "Some streams have invalid JSON schemas",
            errors=errors,
        )
    return ValidationResult.success("All stream schemas are valid JSON Schema Draft 7")


def validate_cursors_exist_in_schema(catalog: AirbyteCatalog) -> ValidationResult:
    """Validate that all defined cursor fields exist in their stream schemas.

    Args:
        catalog: The discovered catalog to validate.

    Returns:
        ValidationResult indicating success or failure.
    """
    errors = []
    for stream in catalog.streams:
        if not stream.default_cursor_field:
            continue

        schema = stream.json_schema
        if "properties" not in schema:
            errors.append(
                f"Stream '{stream.name}' has cursor field but no 'properties' in schema"
            )
            continue

        cursor_path = "/properties/".join(stream.default_cursor_field)
        cursor_field_location = dpath.util.search(schema["properties"], cursor_path)
        if not cursor_field_location:
            errors.append(
                f"Stream '{stream.name}': cursor field {stream.default_cursor_field} "
                "not found in schema properties"
            )

    if errors:
        return ValidationResult.failure(
            "Some cursor fields are not defined in their schemas",
            errors=errors,
        )
    return ValidationResult.success("All cursor fields exist in their schemas")


def _find_all_values_for_key(
    schema: dict[str, Any] | list[Any] | Any,
    key: str,
) -> list[Any]:
    """Find all values for a given key in a nested structure.

    Args:
        schema: The schema or nested structure to search.
        key: The key to search for.

    Returns:
        List of all values found for the key.
    """
    results = []
    if isinstance(schema, dict):
        for k, v in schema.items():
            if k == key:
                results.append(v)
            results.extend(_find_all_values_for_key(v, key))
    elif isinstance(schema, list):
        for item in schema:
            results.extend(_find_all_values_for_key(item, key))
    return results


def validate_no_unresolved_refs(catalog: AirbyteCatalog) -> ValidationResult:
    """Validate that no stream schemas contain unresolved $ref values.

    Args:
        catalog: The discovered catalog to validate.

    Returns:
        ValidationResult indicating success or failure.
    """
    errors = []
    for stream in catalog.streams:
        refs = _find_all_values_for_key(stream.json_schema, "$ref")
        if refs:
            errors.append(f"Stream '{stream.name}' has unresolved $ref values: {refs}")

    if errors:
        return ValidationResult.failure(
            "Some streams have unresolved $ref values",
            errors=errors,
        )
    return ValidationResult.success("No unresolved $ref values found")


def _find_keyword_in_schema(
    schema: dict[str, Any] | list[Any] | str,
    keyword: str,
) -> bool:
    """Find if a keyword exists in a schema, skipping object properties.

    Args:
        schema: The schema to search.
        keyword: The keyword to find.

    Returns:
        True if keyword is found, False otherwise.
    """

    def _find_keyword(
        schema: dict[str, Any] | list[Any] | str,
        key: str,
        skip: bool = False,
    ) -> None:
        if isinstance(schema, list):
            for v in schema:
                _find_keyword(v, key)
        elif isinstance(schema, dict):
            for k, v in schema.items():
                if k == key and not skip:
                    raise StopIteration
                rec_skip = k == "properties" and schema.get("type") == "object"
                _find_keyword(v, key, rec_skip)

    try:
        _find_keyword(schema, keyword)
    except StopIteration:
        return True
    return False


def validate_no_disallowed_keywords(
    catalog: AirbyteCatalog,
    keywords: list[str] | None = None,
) -> ValidationResult:
    """Validate that no stream schemas contain disallowed keywords.

    Args:
        catalog: The discovered catalog to validate.
        keywords: List of disallowed keywords. Defaults to ["allOf", "not"].

    Returns:
        ValidationResult indicating success or failure.
    """
    if keywords is None:
        keywords = ["allOf", "not"]

    errors = []
    for stream in catalog.streams:
        for keyword in keywords:
            if _find_keyword_in_schema(stream.json_schema, keyword):
                errors.append(
                    f"Stream '{stream.name}' contains disallowed keyword '{keyword}'"
                )

    if errors:
        return ValidationResult.failure(
            f"Some streams contain disallowed keywords: {keywords}",
            errors=errors,
        )
    return ValidationResult.success("No disallowed keywords found in schemas")


def validate_primary_keys_exist_in_schema(catalog: AirbyteCatalog) -> ValidationResult:
    """Validate that all primary keys are present in their stream schemas.

    Args:
        catalog: The discovered catalog to validate.

    Returns:
        ValidationResult indicating success or failure.
    """
    errors = []
    for stream in catalog.streams:
        for pk in stream.source_defined_primary_key or []:
            schema = stream.json_schema
            if "properties" not in schema:
                errors.append(
                    f"Stream '{stream.name}' has primary key but no 'properties' in schema"
                )
                continue

            pk_path = "/properties/".join(pk)
            pk_field_location = dpath.util.search(schema["properties"], pk_path)
            if not pk_field_location:
                errors.append(
                    f"Stream '{stream.name}': primary key {pk} not found in schema"
                )

    if errors:
        return ValidationResult.failure(
            "Some primary keys are not defined in their schemas",
            errors=errors,
        )
    return ValidationResult.success("All primary keys exist in their schemas")


def validate_streams_have_sync_modes(catalog: AirbyteCatalog) -> ValidationResult:
    """Validate that all streams have supported_sync_modes defined.

    Args:
        catalog: The discovered catalog to validate.

    Returns:
        ValidationResult indicating success or failure.
    """
    errors = []
    for stream in catalog.streams:
        if stream.supported_sync_modes is None:
            errors.append(
                f"Stream '{stream.name}' is missing supported_sync_modes field"
            )
        elif len(stream.supported_sync_modes) == 0:
            errors.append(f"Stream '{stream.name}' has empty supported_sync_modes list")

    if errors:
        return ValidationResult.failure(
            "Some streams are missing sync mode declarations",
            errors=errors,
        )
    return ValidationResult.success("All streams have sync modes defined")


def validate_additional_properties_is_true(catalog: AirbyteCatalog) -> ValidationResult:
    """Validate that additionalProperties is always true when set.

    Setting additionalProperties to false introduces risk of breaking changes
    when removing properties from the schema.

    Args:
        catalog: The discovered catalog to validate.

    Returns:
        ValidationResult indicating success or failure.
    """
    errors = []
    for stream in catalog.streams:
        additional_props = _find_all_values_for_key(
            stream.json_schema, "additionalProperties"
        )
        for value in additional_props:
            if value is not True:
                errors.append(
                    f"Stream '{stream.name}' has additionalProperties={value}, "
                    "should be true for backward compatibility"
                )

    if errors:
        return ValidationResult.failure(
            "Some streams have additionalProperties set to false",
            errors=errors,
        )
    return ValidationResult.success(
        "All additionalProperties values are true (or not set)"
    )


def validate_stream(stream: AirbyteStream) -> list[ValidationResult]:
    """Run all validations on a single stream.

    Args:
        stream: The stream to validate.

    Returns:
        List of ValidationResult objects.
    """
    catalog = AirbyteCatalog(streams=[stream])
    return [
        validate_schemas_are_valid_json_schema(catalog),
        validate_cursors_exist_in_schema(catalog),
        validate_no_unresolved_refs(catalog),
        validate_no_disallowed_keywords(catalog),
        validate_primary_keys_exist_in_schema(catalog),
        validate_streams_have_sync_modes(catalog),
        validate_additional_properties_is_true(catalog),
    ]


def validate_catalog(catalog: AirbyteCatalog) -> list[ValidationResult]:
    """Run all catalog validations.

    Args:
        catalog: The catalog to validate.

    Returns:
        List of ValidationResult objects.
    """
    return [
        validate_catalog_has_streams(catalog),
        validate_no_duplicate_stream_names(catalog),
        validate_schemas_are_valid_json_schema(catalog),
        validate_cursors_exist_in_schema(catalog),
        validate_no_unresolved_refs(catalog),
        validate_no_disallowed_keywords(catalog),
        validate_primary_keys_exist_in_schema(catalog),
        validate_streams_have_sync_modes(catalog),
        validate_additional_properties_is_true(catalog),
    ]



# Copyright (c) 2025 Airbyte, Inc., all rights reserved.
"""Tests for the connector_stubs module.

These tests focus on pure logic that can be tested without mocks:
- ConnectorStub model validation
- find_stub_by_connector matching logic
- Local file operations (using temp directories for round-trip testing)
"""

from pathlib import Path

import pytest

from airbyte_ops_mcp.registry._gcs_util import (
    DEV_METADATA_SERVICE_BUCKET_NAME,
    PROD_METADATA_SERVICE_BUCKET_NAME,
    get_bucket_name,
)
from airbyte_ops_mcp.registry.connector_stubs import (
    CONNECTOR_STUBS_FILE,
    ConnectorStub,
    find_stub_by_connector,
    load_local_stubs,
    save_local_stubs,
)


@pytest.mark.parametrize(
    ("env", "expected_bucket"),
    [
        pytest.param("prod", PROD_METADATA_SERVICE_BUCKET_NAME, id="prod"),
        pytest.param("dev", DEV_METADATA_SERVICE_BUCKET_NAME, id="dev"),
    ],
)
def test_get_bucket_name(env: str, expected_bucket: str) -> None:
    """Test bucket name selection."""
    assert get_bucket_name(env) == expected_bucket


@pytest.mark.parametrize(
    ("stub_data", "expected_id", "expected_optional_none"),
    [
        pytest.param(
            {
                "id": "source-test",
                "name": "Test",
                "url": "https://docs.airbyte.com/test",
                "icon": "https://storage.googleapis.com/test/icon.svg",
            },
            "source-test",
            ["definition_id", "label", "type", "codename"],
            id="required_fields_only",
        ),
        pytest.param(
            {
                "id": "source-oracle-enterprise",
                "name": "Oracle",
                "url": "https://docs.airbyte.com/oracle",
                "icon": "https://storage.googleapis.com/icons/oracle.svg",
                "definition_id": "196a42fc-39f2-473f-88ff-d68b2ea702e9",
                "label": "enterprise",
                "type": "enterprise_source",
                "codename": "oracle-ent",
            },
            "source-oracle-enterprise",
            [],
            id="all_fields",
        ),
    ],
)
def test_connector_stub_model(
    stub_data: dict, expected_id: str, expected_optional_none: list[str]
) -> None:
    """Test ConnectorStub model validation and field handling."""
    stub = ConnectorStub(**stub_data)
    assert stub.id == expected_id
    for field in expected_optional_none:
        assert getattr(stub, field) is None


@pytest.mark.parametrize(
    ("stubs", "connector", "expected_id"),
    [
        pytest.param(
            [{"id": "source-oracle-enterprise", "name": "Oracle"}],
            "source-oracle-enterprise",
            "source-oracle-enterprise",
            id="exact_id_match",
        ),
        pytest.param(
            [{"id": "source-oracle-enterprise", "name": "Oracle"}],
            "source-oracle",
            "source-oracle-enterprise",
            id="enterprise_suffix_match",
        ),
        pytest.param(
            [{"id": "source-sap-hana", "name": "SAP HANA"}],
            "sap-hana",
            "source-sap-hana",
            id="name_match",
        ),
        pytest.param(
            [{"id": "source-oracle-enterprise", "name": "Oracle"}],
            "source-nonexistent",
            None,
            id="no_match",
        ),
    ],
)
def test_find_stub_by_connector(
    stubs: list[dict], connector: str, expected_id: str | None
) -> None:
    """Test stub lookup by various matching strategies."""
    result = find_stub_by_connector(stubs, connector)
    if expected_id is None:
        assert result is None
    else:
        assert result is not None
        assert result["id"] == expected_id


def test_local_stubs_round_trip(tmp_path: Path) -> None:
    """Test saving and loading local stubs (round-trip without mocks)."""
    test_stubs = [
        {
            "id": "source-test-enterprise",
            "name": "Test Connector",
            "url": "https://docs.airbyte.com/test",
            "icon": "https://storage.googleapis.com/test/icon.svg",
            "label": "enterprise",
        },
        {
            "id": "source-another",
            "name": "Another",
            "url": "https://docs.airbyte.com/another",
            "icon": "https://storage.googleapis.com/another/icon.svg",
        },
    ]

    save_local_stubs(tmp_path, test_stubs)
    loaded = load_local_stubs(tmp_path)

    assert loaded == test_stubs
    assert (tmp_path / CONNECTOR_STUBS_FILE).exists()


@pytest.mark.parametrize(
    ("file_content", "expected_error"),
    [
        pytest.param(None, FileNotFoundError, id="file_not_found"),
        pytest.param('{"not": "a list"}', ValueError, id="invalid_json_structure"),
    ],
)
def test_load_local_stubs_errors(
    tmp_path: Path, file_content: str | None, expected_error: type
) -> None:
    """Test error handling when loading local stubs."""
    if file_content is not None:
        (tmp_path / CONNECTOR_STUBS_FILE).write_text(file_content)

    with pytest.raises(expected_error):
        load_local_stubs(tmp_path)



#
# Copyright (c) 2023 Airbyte, Inc., all rights reserved.
#

import json
from pathlib import Path
from typing import Tuple

from consts import GCS_PUBLIC_DOMAIN
from google.cloud import storage  # type: ignore
from google.oauth2 import service_account  # type: ignore
from pipelines import main_logger


def upload_to_gcs(
    file_path: Path, bucket_name: str, object_name: str, credentials: str
) -> Tuple[str, str]:
    """Upload a file to a GCS bucket.

    Args:
        file_path (Path): The path to the file to upload.
        bucket_name (str): The name of the GCS bucket.
        object_name (str): The name of the object in the GCS bucket.
        credentials (str): The GCS credentials as a JSON string.
    """
    # Exit early if file does not exist
    if not file_path.exists():
        main_logger.warning(f"File {file_path} does not exist. Skipping upload to GCS.")
        return "", ""

    credentials = service_account.Credentials.from_service_account_info(
        json.loads(credentials)
    )
    client = storage.Client(credentials=credentials)
    bucket = client.get_bucket(bucket_name)
    blob = bucket.blob(object_name)
    blob.upload_from_filename(str(file_path))
    gcs_uri = f"gs://{bucket_name}/{object_name}"
    public_url = f"{GCS_PUBLIC_DOMAIN}/{bucket_name}/{object_name}"
    return gcs_uri, public_url


def sanitize_gcp_credentials(raw_value: str) -> str:
    """Try to parse the raw string input that should contain a json object with the GCS credentials.
    It will raise an exception if the parsing fails and help us to fail fast on invalid credentials input.

    Args:
        raw_value (str): A string representing a json object with the GCS credentials.

    Returns:
        str: The raw value string if it was successfully parsed.
    """
    return json.dumps(json.loads(raw_value))



# #
# # Copyright (c) 2023 Airbyte, Inc., all rights reserved.
# #

# import importlib.util
# from importlib import metadata
# from importlib.abc import Loader

# from dagger import Container

# from pipelines.airbyte_ci.connectors.context import ConnectorContext


# def get_dagger_sdk_version() -> str:
#     try:
#         return metadata.version("dagger-io")
#     except metadata.PackageNotFoundError:
#         return "n/a"


# async def finalize_build(context: ConnectorContext, connector_container: Container) -> Container:
#     """Finalize build by adding dagger engine version label and running finalize_build.sh or finalize_build.py if present in the connector directory."""
#     original_user = (await connector_container.with_exec(["whoami"]).stdout()).strip()
#     # Switch to root to finalize the build with superuser permissions
#     connector_container = connector_container.with_user("root")
#     connector_container = connector_container.with_label("io.dagger.engine_version", get_dagger_sdk_version())
#     connector_dir_with_finalize_script = await context.get_connector_dir(include=["finalize_build.sh", "finalize_build.py"])
#     finalize_scripts = await connector_dir_with_finalize_script.entries()
#     if not finalize_scripts:
#         return connector_container

#     # We don't want finalize scripts to override the entrypoint so we keep it in memory to reset it after finalization
#     original_entrypoint = await connector_container.entrypoint()
#     if not original_entrypoint:
#         original_entrypoint = []

#     has_finalize_bash_script = "finalize_build.sh" in finalize_scripts
#     has_finalize_python_script = "finalize_build.py" in finalize_scripts
#     if has_finalize_python_script and has_finalize_bash_script:
#         raise Exception("Connector has both finalize_build.sh and finalize_build.py, please remove one of them")

#     if has_finalize_python_script:
#         context.logger.info(f"{context.connector.technical_name} has a finalize_build.py script, running it to finalize build...")
#         module_path = context.connector.code_directory / "finalize_build.py"
#         connector_finalize_module_spec = importlib.util.spec_from_file_location(
#             f"{context.connector.code_directory.name}_finalize", module_path
#         )
#         if connector_finalize_module_spec is None:
#             raise Exception("Connector has a finalize_build.py script but it can't be loaded.")
#         connector_finalize_module = importlib.util.module_from_spec(connector_finalize_module_spec)
#         if not isinstance(connector_finalize_module_spec.loader, Loader):
#             raise Exception("Connector has a finalize_build.py script but it can't be loaded.")
#         connector_finalize_module_spec.loader.exec_module(connector_finalize_module)
#         try:
#             connector_container = await connector_finalize_module.finalize_build(context, connector_container)
#         except AttributeError:
#             raise Exception("Connector has a finalize_build.py script but it doesn't have a finalize_build function.")

#     if has_finalize_bash_script:
#         context.logger.info(f"{context.connector.technical_name} has finalize_build.sh script, running it to finalize build...")
#         connector_container = (
#             connector_container.with_file("/tmp/finalize_build.sh", connector_dir_with_finalize_script.file("finalize_build.sh"))
#             .with_entrypoint(["sh"])
#             .with_exec(["/tmp/finalize_build.sh"], use_entrypoint=True)
#         )
#     # Switch back to the original user
#     connector_container = connector_container.with_exec(["chown", "-R", f"{original_user}:{original_user}", "/tmp"])
#     connector_container = connector_container.with_user(original_user)
#     return connector_container.with_entrypoint(original_entrypoint)



# Copyright (c) 2025 Airbyte, Inc., all rights reserved.
"""Analyze and generate Python connector dependency metadata.

This module replaces the legacy `upload-python-dependencies.sh` script by
providing functions that:

* Detect whether a connector is Python-based (via metadata tags).
* Run `pip freeze` inside the connector's Docker image to capture the
  full dependency tree.
* Build and write a `dependencies.json` artifact matching the format
  expected by downstream consumers (e.g. the compile step's
  `packageInfo.cdk_version` extraction).
* Extract the `airbyte-cdk` version from a dependencies dict.
"""

from __future__ import annotations

import json
import logging
import subprocess
from datetime import datetime, timezone
from pathlib import Path
from typing import Any

from airbyte_ops_mcp.registry._constants import (
    CONNECTOR_DEPENDENCY_FILE_NAME,
)

logger = logging.getLogger(__name__)

PYTHON_CDK_SLUG = "python"


def _is_python_connector(metadata_data: dict[str, Any]) -> bool:
    """Check if a connector is Python-based using its metadata tags.

    Returns `True` when the metadata `tags` list includes a
    `language:python` entry.
    """
    tags: list[str] = metadata_data.get("tags", [])
    return any(tag == "language:python" for tag in tags)


def _run_docker_pip_freeze(docker_image: str) -> list[dict[str, str]]:
    """Run `pip freeze` inside the connector Docker image.

    Returns a list of `{"package_name": ..., "version": ...}` dicts,
    matching the format produced by the legacy
    `upload-python-dependencies.sh` script.

    Raises:
        RuntimeError: If the docker command fails.
    """
    cmd = [
        "docker",
        "run",
        "--rm",
        "--entrypoint",
        "pip",
        docker_image,
        "freeze",
    ]
    logger.info("Running: %s", " ".join(cmd))

    result = subprocess.run(
        cmd,
        capture_output=True,
        text=True,
        timeout=120,
    )

    if result.returncode != 0:
        raise RuntimeError(
            f"docker pip freeze failed with exit code {result.returncode}: "
            f"{result.stderr.strip()}"
        )

    dependencies: list[dict[str, str]] = []
    for line in result.stdout.strip().splitlines():
        line = line.split("#")[0].strip()
        if not line or "==" not in line:
            continue
        parts = line.split("==", 1)
        if len(parts) == 2:
            dependencies.append({"package_name": parts[0], "version": parts[1]})
    return dependencies


def _build_dependencies_json(
    connector_name: str,
    version: str,
    docker_repository: str,
    definition_id: str,
    dependencies: list[dict[str, str]],
) -> dict[str, Any]:
    """Build the `dependencies.json` payload.

    The structure matches the legacy `upload-python-dependencies.sh`
    output so that downstream consumers (e.g. the compile step's
    `packageInfo.cdk_version` extraction) work without changes.
    """
    now = datetime.now(tz=timezone.utc)
    generation_time = now.strftime("%Y-%m-%dT%H:%M:%S.%f")
    return {
        "connector_technical_name": connector_name,
        "connector_repository": docker_repository,
        "connector_version": version,
        "connector_definition_id": definition_id,
        "dependencies": dependencies,
        "generation_time": generation_time,
    }


def generate_python_dependencies_file(
    metadata_data: dict[str, Any],
    docker_image: str,
    output_dir: Path,
) -> dict[str, Any] | None:
    """Generate `dependencies.json` for a Python connector.

    Runs `pip freeze` inside the connector's Docker image and writes
    the result to `output_dir`.  Returns the parsed dependencies dict
    (for use by `_apply_package_info_fields`) or `None` on failure.
    """
    docker_repo = metadata_data.get("dockerRepository", "")
    connector_name = docker_repo.replace("airbyte/", "")
    version = metadata_data.get("dockerImageTag", "")
    definition_id = metadata_data.get("definitionId", "")

    try:
        dependencies = _run_docker_pip_freeze(docker_image)
    except (RuntimeError, subprocess.TimeoutExpired) as exc:
        logger.warning("Could not generate dependencies.json: %s", exc)
        return None

    if not dependencies:
        logger.warning("pip freeze returned no dependencies for %s.", docker_image)
        return None

    deps_json = _build_dependencies_json(
        connector_name=connector_name,
        version=version,
        docker_repository=docker_repo,
        definition_id=definition_id,
        dependencies=dependencies,
    )

    out_path = output_dir / CONNECTOR_DEPENDENCY_FILE_NAME
    out_path.write_text(json.dumps(deps_json, indent=2) + "\n")
    logger.info("Wrote %s (%d dependencies)", out_path, len(dependencies))
    return deps_json


def extract_cdk_version_from_dependencies(
    dependencies_json: dict[str, Any],
) -> str | None:
    """Extract the `airbyte-cdk` version string from a dependencies dict.

    Returns a string like `"python:1.2.3"` or `None` if not found.
    """
    for package in dependencies_json.get("dependencies", []):
        if package.get("package_name") == "airbyte-cdk":
            pkg_ver = package.get("version")
            return f"{PYTHON_CDK_SLUG}:{pkg_ver}" if pkg_ver else None
    return None



# Copyright (c) 2023 Airbyte, Inc., all rights reserved.
from __future__ import annotations

from airbyte_ops_mcp._legacy.airbyte_ci.connector_qa import consts
from airbyte_ops_mcp._legacy.airbyte_ci.connector_qa.checks import packaging
from airbyte_ops_mcp._legacy.airbyte_ci.connector_qa.models import CheckStatus


class TestCheckConnectorUsesPoetry:
    def test_fail_when_pyproject_toml_file_does_not_exist(self, tmp_path, mocker):
        # Arrange
        connector = mocker.MagicMock(code_directory=tmp_path)

        # Act
        result = packaging.CheckConnectorUsesPoetry()._run(connector)

        # Assert
        assert result.status == CheckStatus.FAILED
        assert result.message == f"{consts.PYPROJECT_FILE_NAME} file is missing"

    def test_fail_when_poetry_lock_file_does_not_exist(self, tmp_path, mocker):
        # Arrange
        connector = mocker.MagicMock(code_directory=tmp_path)
        (tmp_path / consts.PYPROJECT_FILE_NAME).touch()

        # Act
        result = packaging.CheckConnectorUsesPoetry()._run(connector)

        # Assert
        assert result.status == CheckStatus.FAILED
        assert result.message == f"{consts.POETRY_LOCK_FILE_NAME} file is missing"

    def test_fail_when_setup_py_file_exists(self, tmp_path, mocker):
        # Arrange
        connector = mocker.MagicMock(code_directory=tmp_path)
        (tmp_path / consts.PYPROJECT_FILE_NAME).touch()
        (tmp_path / consts.POETRY_LOCK_FILE_NAME).touch()
        (tmp_path / consts.SETUP_PY_FILE_NAME).touch()

        # Act
        result = packaging.CheckConnectorUsesPoetry()._run(connector)

        # Assert
        assert result.status == CheckStatus.FAILED
        assert (
            result.message
            == f"{consts.SETUP_PY_FILE_NAME} file exists. Please remove it and use {consts.PYPROJECT_FILE_NAME} instead"
        )

    def test_pass_when_poetry_is_used(self, tmp_path, mocker):
        # Arrange
        connector = mocker.MagicMock(code_directory=tmp_path)
        (tmp_path / consts.PYPROJECT_FILE_NAME).touch()
        (tmp_path / consts.POETRY_LOCK_FILE_NAME).touch()

        # Act
        result = packaging.CheckConnectorUsesPoetry()._run(connector)

        # Assert
        assert result.status == CheckStatus.PASSED
        assert result.message == "Poetry is used for dependency management"


class TestCheckManifestOnlyConnectorBaseImage:
    def test_pass_with_source_declarative_manifest(self, mocker, tmp_path):
        connector = mocker.MagicMock(
            code_directory=tmp_path,
            metadata={
                "connectorBuildOptions": {
                    "baseImage": "docker.io/airbyte/source-declarative-manifest:4.3.0@SHA"
                }
            },
        )

        # Act
        result = packaging.CheckManifestOnlyConnectorBaseImage()._run(connector)

        # Assert
        assert result.status == CheckStatus.PASSED
        assert "Connector uses source-declarative-manifest base image" in result.message

    def test_fail_with_different_image(self, mocker, tmp_path):
        connector = mocker.MagicMock(
            code_directory=tmp_path,
            metadata={
                "connectorBuildOptions": {
                    "baseImage": "docker.io/airbyte/connector-base-image:2.0.0@SHA"
                }
            },
        )

        # Act
        result = packaging.CheckManifestOnlyConnectorBaseImage()._run(connector)

        # Assert
        assert result.status == CheckStatus.FAILED
        assert (
            "A manifest-only connector must use `source-declarative-manifest` base image"
            in result.message
        )

    def test_fail_with_missing_image(self, mocker, tmp_path):
        connector = mocker.MagicMock(
            code_directory=tmp_path,
            metadata={"connectorBuildOptions": {}},
        )

        # Act
        result = packaging.CheckManifestOnlyConnectorBaseImage()._run(connector)

        # Assert
        assert result.status == CheckStatus.FAILED
        assert (
            "A manifest-only connector must use `source-declarative-manifest` base image"
            in result.message
        )


class TestCheckPublishToPyPiIsDeclared:
    def test_fail_if_publish_to_pypi_is_not_declared(self, mocker):
        # Arrange
        connector = mocker.MagicMock(metadata={"remoteRegistries": {}})

        # Act
        result = packaging.CheckPublishToPyPiIsDeclared()._run(connector)

        # Assert
        assert result.status == CheckStatus.FAILED
        assert "PyPi publishing is not declared" in result.message

    def test_pass_if_publish_to_pypi_is_enabled(self, mocker):
        # Arrange
        connector = mocker.MagicMock(
            metadata={"remoteRegistries": {"pypi": {"enabled": True}}}
        )

        # Act
        result = packaging.CheckPublishToPyPiIsDeclared()._run(connector)

        # Assert
        assert result.status == CheckStatus.PASSED
        assert "PyPi publishing is declared" in result.message

    def test_pass_if_publish_to_pypi_is_disabled(self, mocker):
        # Arrange
        connector = mocker.MagicMock(
            metadata={"remoteRegistries": {"pypi": {"enabled": False}}}
        )

        # Act
        result = packaging.CheckPublishToPyPiIsDeclared()._run(connector)

        # Assert
        assert result.status == CheckStatus.PASSED
        assert "PyPi publishing is declared" in result.message


class TestCheckConnectorLicense:
    def test_fail_when_license_is_missing(self, mocker):
        # Arrange
        connector = mocker.MagicMock(metadata={})

        # Act
        result = packaging.CheckConnectorLicense()._run(connector)

        # Assert
        assert result.status == CheckStatus.FAILED
        assert "License is missing in the metadata file" in result.message

    def test_fail_when_license_is_not_valid(self, mocker):
        # Arrange
        connector = mocker.MagicMock(metadata={"license": "MITO"})

        # Act
        result = packaging.CheckConnectorLicense()._run(connector)

        # Assert
        assert result.status == CheckStatus.FAILED
        assert "Connector is not using a valid license" in result.message


class TestCheckConnectorLicenseMatchInPyproject:
    def test_fail_when_missing_metadata_license(self, mocker):
        # Arrange
        connector = mocker.MagicMock(metadata={})

        # Act
        result = packaging.CheckConnectorLicenseMatchInPyproject()._run(connector)

        # Assert
        assert result.status == CheckStatus.FAILED
        assert (
            f"License is missing in the {consts.METADATA_FILE_NAME} file"
            in result.message
        )

    def test_fail_when_missing_pyproject_toml_file(self, mocker, tmp_path):
        # Arrange
        connector = mocker.MagicMock(
            metadata={"license": "MIT"}, code_directory=tmp_path
        )

        # Act
        result = packaging.CheckConnectorLicenseMatchInPyproject()._run(connector)

        # Assert
        assert result.status == CheckStatus.FAILED
        assert f"{consts.PYPROJECT_FILE_NAME} file is missing" in result.message

    def test_fail_when_unparseable_toml_file(self, mocker, tmp_path):
        # Arrange
        connector = mocker.MagicMock(
            metadata={"license": "MIT"}, code_directory=tmp_path
        )
        (tmp_path / consts.PYPROJECT_FILE_NAME).write_text("abc")

        # Act
        result = packaging.CheckConnectorLicenseMatchInPyproject()._run(connector)

        # Assert
        assert result.status == CheckStatus.FAILED
        assert f"{consts.PYPROJECT_FILE_NAME} is invalid toml file" in result.message

    def test_fail_when_missing_poetry_license(self, mocker, tmp_path):
        # Arrange
        connector = mocker.MagicMock(
            metadata={"license": "MIT"}, code_directory=tmp_path
        )
        (tmp_path / consts.PYPROJECT_FILE_NAME).write_text("[tool.poetry]")

        # Act
        result = packaging.CheckConnectorLicenseMatchInPyproject()._run(connector)

        # Assert
        assert result.status == CheckStatus.FAILED
        assert (
            f"Connector is missing license in {consts.PYPROJECT_FILE_NAME}. Please add it"
            in result.message
        )

    def test_fail_when_license_mismatch(self, mocker, tmp_path):
        # Arrange
        connector = mocker.MagicMock(
            metadata={"license": "MIT"}, code_directory=tmp_path
        )
        (tmp_path / consts.PYPROJECT_FILE_NAME).write_text(
            '[tool.poetry]\nlicense = "Elv2"'
        )

        # Act
        result = packaging.CheckConnectorLicenseMatchInPyproject()._run(connector)

        # Assert
        assert result.status == CheckStatus.FAILED
        assert (
            f"Connector is licensed under Elv2 in {consts.PYPROJECT_FILE_NAME}, but licensed under MIT in {consts.METADATA_FILE_NAME}. These two files have to be consistent"
            in result.message
        )

    def test_pass_when_license_match(self, mocker, tmp_path):
        # Arrange
        connector = mocker.MagicMock(
            metadata={"license": "ELV2"}, code_directory=tmp_path
        )
        (tmp_path / consts.PYPROJECT_FILE_NAME).write_text(
            '[tool.poetry]\nlicense = "Elv2"'
        )

        # Act
        result = packaging.CheckConnectorLicenseMatchInPyproject()._run(connector)

        # Assert
        assert result.status == CheckStatus.PASSED
        assert (
            f"License in {consts.METADATA_FILE_NAME} and {consts.PYPROJECT_FILE_NAME} file match"
            in result.message
        )


class TestCheckConnectorVersionMatchInPyproject:
    def test_fail_when_missing_metadata_docker_image_tag(self, mocker):
        # Arrange
        connector = mocker.MagicMock(metadata={})

        # Act
        result = packaging.CheckConnectorVersionMatchInPyproject()._run(connector)

        # Assert
        assert result.status == CheckStatus.FAILED
        assert f"dockerImageTag is missing in {consts.METADATA_FILE_NAME}"

    def test_fail_when_missing_pyproject_toml_file(self, mocker, tmp_path):
        # Arrange
        connector = mocker.MagicMock(
            metadata={"dockerImageTag": "0.0.0"}, code_directory=tmp_path
        )

        # Act
        result = packaging.CheckConnectorVersionMatchInPyproject()._run(connector)

        # Assert
        assert result.status == CheckStatus.FAILED
        assert f"{consts.PYPROJECT_FILE_NAME} file is missing" in result.message

    def test_fail_when_unparseable_toml_file(self, mocker, tmp_path):
        # Arrange
        connector = mocker.MagicMock(
            metadata={"dockerImageTag": "0.0.0"}, code_directory=tmp_path
        )
        (tmp_path / consts.PYPROJECT_FILE_NAME).write_text("abc")

        # Act
        result = packaging.CheckConnectorVersionMatchInPyproject()._run(connector)

        # Assert
        assert result.status == CheckStatus.FAILED
        assert f"{consts.PYPROJECT_FILE_NAME} is invalid toml file" in result.message

    def test_fail_when_missing_poetry_version(self, mocker, tmp_path):
        # Arrange
        connector = mocker.MagicMock(
            metadata={"dockerImageTag": "0.0.0"}, code_directory=tmp_path
        )
        (tmp_path / consts.PYPROJECT_FILE_NAME).write_text("[tool.poetry]")

        # Act
        result = packaging.CheckConnectorVersionMatchInPyproject()._run(connector)

        # Assert
        assert result.status == CheckStatus.FAILED
        assert (
            f"Version field is missing in the {consts.PYPROJECT_FILE_NAME} file"
            in result.message
        )

    def test_fail_when_version_mismatch(self, mocker, tmp_path):
        # Arrange
        connector = mocker.MagicMock(
            metadata={"dockerImageTag": "0.0.0"}, code_directory=tmp_path
        )
        (tmp_path / consts.PYPROJECT_FILE_NAME).write_text(
            '[tool.poetry]\nversion = "1.0.0"'
        )

        # Act
        result = packaging.CheckConnectorVersionMatchInPyproject()._run(connector)

        # Assert
        assert result.status == CheckStatus.FAILED
        assert (
            f"Version is 0.0.0 in {consts.METADATA_FILE_NAME}, but version is 1.0.0 in {consts.PYPROJECT_FILE_NAME}. These two files have to be consistent"
            in result.message
        )

    def test_pass_when_version_match(self, mocker, tmp_path):
        # Arrange
        connector = mocker.MagicMock(
            metadata={"dockerImageTag": "1.0.0"}, code_directory=tmp_path
        )
        (tmp_path / consts.PYPROJECT_FILE_NAME).write_text(
            '[tool.poetry]\nversion = "1.0.0"'
        )

        # Act
        result = packaging.CheckConnectorVersionMatchInPyproject()._run(connector)

        # Assert
        assert result.status == CheckStatus.PASSED
        assert (
            f"Version in {consts.METADATA_FILE_NAME} and {consts.PYPROJECT_FILE_NAME} file match"
            in result.message
        )


class TestCheckVersionFollowsSemver:
    def test_fail_when_missing_metadata_docker_image_tag(self, mocker):
        # Arrange
        connector = mocker.MagicMock(metadata={})

        # Act
        result = packaging.CheckVersionFollowsSemver()._run(connector)

        # Assert
        assert result.status == CheckStatus.FAILED
        assert f"dockerImageTag is missing in {consts.METADATA_FILE_NAME}"

    def test_fail_when_version_is_not_semver(self, mocker):
        # Arrange
        connector = mocker.MagicMock(metadata={"dockerImageTag": "1.1"})

        # Act
        result = packaging.CheckVersionFollowsSemver()._run(connector)

        # Assert
        assert result.status == CheckStatus.FAILED
        assert (
            f"Connector version {connector.metadata['dockerImageTag']} does not follow semantic versioning"
            in result.message
        )

    def test_pass_when_version_follows_semver(self, mocker):
        # Arrange
        connector = mocker.MagicMock(metadata={"dockerImageTag": "1.1.1"})

        # Act
        result = packaging.CheckVersionFollowsSemver()._run(connector)

        # Assert
        assert result.status == CheckStatus.PASSED
        assert "Connector version follows semantic versioning" in result.message



"""Airbyte Admin MCP - MCP and API interfaces that let the agents do the admin work.

.. include:: ../../../README.md
"""

from airbyte_ops_mcp import (
    airbyte_repo,
    cli,
    cloud_admin,
    connection_config_retriever,
    constants,
    mcp,
    prod_db_access,
    registry,
    regression_tests,
)

__version__ = "0.1.0"


def hello() -> str:
    """Return a friendly greeting."""
    return "Hello from airbyte-internal-ops!"


def get_version() -> str:
    """Return the current version."""
    return __version__


__all__ = [
    "__version__",
    "airbyte_repo",
    "cli",
    "cloud_admin",
    "connection_config_retriever",
    "constants",
    "get_version",
    "hello",
    "mcp",
    "prod_db_access",
    "registry",
    "regression_tests",
]



# Copyright (c) 2025 Airbyte, Inc., all rights reserved.
"""MCP tools for customer tier lookup and cache management.

Provides tools to resolve customer tiers for organizations, workspaces, and connections,
and to manage the tier cache.
"""

from __future__ import annotations

from typing import Annotated, Any

from fastmcp import FastMCP
from fastmcp_extensions import mcp_tool, register_mcp_tools
from pydantic import BaseModel, Field

from airbyte_ops_mcp.prod_db_access.queries import query_connection_workspace_details
from airbyte_ops_mcp.tier_cache import (
    CustomerTier,
    TierCacheStats,
    TierSummary,
    build_tier_summary,
    get_cache_stats,
    get_org_tier,
    get_org_tiers,
    refresh_tier_cache,
    resolve_workspaces,
)

# =============================================================================
# Pydantic Models for MCP Tool Responses
# =============================================================================


class TierLookupEntry(BaseModel):
    """A single resolved tier entry for an org, workspace, or connection."""

    input_id: str = Field(description="The original ID that was looked up")
    input_type: str = Field(
        description="Type of the input ID: 'organization', 'workspace', or 'connection'"
    )
    organization_id: str | None = Field(
        default=None, description="Resolved organization UUID"
    )
    workspace_id: str | None = Field(
        default=None,
        description="Workspace UUID (if input was workspace or connection)",
    )
    connection_id: str | None = Field(
        default=None, description="Connection UUID (if input was connection)"
    )
    customer_tier: CustomerTier = Field(
        description="Resolved tier: TIER_0, TIER_1, or TIER_2"
    )
    dataplane_name: str | None = Field(
        default=None, description="Dataplane region name (e.g., 'US', 'EU')"
    )
    is_eu: bool = Field(
        default=False, description="Whether the entity is in the EU region"
    )
    resolved: bool = Field(
        default=True, description="Whether the ID was successfully resolved"
    )


class TierLookupResult(BaseModel):
    """Result of a customer tier lookup across multiple IDs."""

    entries: list[TierLookupEntry] = Field(
        description="Individual tier lookup results for each input ID"
    )
    summary: TierSummary = Field(
        description="Tier distribution summary across all resolved entries"
    )
    summary_text: str = Field(description="Human-readable tier distribution summary")


class TierCacheRefreshResult(BaseModel):
    """Result of refreshing the tier cache."""

    stats: TierCacheStats = Field(description="Cache statistics after refresh")
    message: str = Field(description="Human-readable result message")


# =============================================================================
# MCP Tools
# =============================================================================


@mcp_tool(
    read_only=True,
    idempotent=True,
)
def lookup_customer_tiers(
    organization_ids: Annotated[
        list[str] | None,
        Field(
            description=(
                "List of organization UUIDs to look up tiers for. "
                "Example: ['664c690e-5263-49ba-b01f-4a6759b3330a']"
            ),
            default=None,
        ),
    ] = None,
    workspace_ids: Annotated[
        list[str] | None,
        Field(
            description=(
                "List of workspace UUIDs to look up tiers for. "
                "Each workspace will be resolved to its organization and tier. "
                "Example: ['266ebdfe-0d7b-4540-9817-de7e4505ba61']"
            ),
            default=None,
        ),
    ] = None,
    connection_ids: Annotated[
        list[str] | None,
        Field(
            description=(
                "List of connection UUIDs to look up tiers for. "
                "Each connection will be resolved to its workspace, organization, and tier."
            ),
            default=None,
        ),
    ] = None,
) -> TierLookupResult:
    """Look up customer tier classification for organizations, workspaces, and/or connections.

    Accepts mixed lists of organization IDs, workspace IDs, and connection IDs.
    Resolves each to its organization and maps to a customer tier (TIER_0, TIER_1, or TIER_2).

    Tier 0 and Tier 1 orgs are explicitly tracked in Salesforce and cached from BigQuery.
    Any org not in the cache defaults to TIER_2.

    Returns enriched entries with tier, region (EU/US), and a summary of the distribution.
    """
    entries: list[TierLookupEntry] = []

    # Resolve organization IDs directly
    org_ids = organization_ids or []
    if org_ids:
        org_results = get_org_tiers(org_ids)
        for result in org_results:
            entries.append(
                TierLookupEntry(
                    input_id=result.organization_id,
                    input_type="organization",
                    organization_id=result.organization_id,
                    customer_tier=result.customer_tier,
                    resolved=True,
                )
            )

    # Resolve workspace IDs -> org -> tier
    ws_ids = workspace_ids or []
    if ws_ids:
        ws_results = resolve_workspaces(ws_ids)
        for result in ws_results:
            entries.append(
                TierLookupEntry(
                    input_id=result.workspace_id,
                    input_type="workspace",
                    organization_id=result.organization_id,
                    workspace_id=result.workspace_id,
                    customer_tier=result.customer_tier,
                    dataplane_name=result.dataplane_name,
                    is_eu=result.is_eu,
                    resolved=result.resolved,
                )
            )

    # Resolve connection IDs -> workspace -> org -> tier
    conn_ids = connection_ids or []
    if conn_ids:
        conn_details = query_connection_workspace_details(conn_ids)
        conn_map: dict[str, dict[str, Any]] = {
            str(row["connection_id"]): row for row in conn_details
        }

        for conn_id in conn_ids:
            row = conn_map.get(conn_id)
            if row is None:
                entries.append(
                    TierLookupEntry(
                        input_id=conn_id,
                        input_type="connection",
                        connection_id=conn_id,
                        customer_tier="TIER_2",
                        resolved=False,
                    )
                )
                continue

            org_id = str(row["organization_id"])
            dataplane_name = row.get("dataplane_name") or "US"
            org_result = get_org_tier(org_id)

            entries.append(
                TierLookupEntry(
                    input_id=conn_id,
                    input_type="connection",
                    organization_id=org_id,
                    workspace_id=str(row["workspace_id"]),
                    connection_id=conn_id,
                    customer_tier=org_result.customer_tier,
                    dataplane_name=dataplane_name,
                    is_eu=dataplane_name == "EU",
                    resolved=True,
                )
            )

    # Build summary from enriched entries
    summary_rows = [{"customer_tier": e.customer_tier} for e in entries if e.resolved]
    summary = build_tier_summary(summary_rows)

    return TierLookupResult(
        entries=entries,
        summary=summary,
        summary_text=str(summary),
    )


@mcp_tool(
    read_only=False,
    idempotent=True,
)
def refresh_customer_tier_cache() -> TierCacheRefreshResult:
    """Force-refresh the customer tier cache from BigQuery.

    The tier cache is automatically refreshed every 24 hours. Use this tool to
    manually trigger a refresh if you need the latest tier data immediately
    (e.g., after a Salesforce update).

    Returns cache statistics after the refresh.
    """
    stats = refresh_tier_cache()
    return TierCacheRefreshResult(
        stats=stats,
        message=(
            f"Tier cache refreshed: {stats.tier_cache_size} orgs cached, "
            f"{stats.workspace_cache_size} workspaces cached."
        ),
    )


@mcp_tool(
    read_only=True,
    idempotent=True,
)
def get_customer_tier_cache_stats() -> TierCacheStats:
    """Get current statistics about the customer tier cache.

    Returns cache size, age, and file paths for both the tier cache
    (org -> tier) and workspace cache (workspace -> org + region).

    Useful for checking cache freshness and diagnosing issues.
    """
    return get_cache_stats()


def register_tier_lookup_tools(app: FastMCP) -> None:
    """Register tier lookup tools with the FastMCP app."""
    register_mcp_tools(app, mcp_module=__name__)



# Copyright (c) 2024 Airbyte, Inc., all rights reserved.
from __future__ import annotations

import logging
from abc import ABC, abstractmethod
from dataclasses import dataclass
from pathlib import Path
from typing import TYPE_CHECKING

from connector_ops.utils import Connector  # type: ignore
from google.cloud import storage  # type: ignore

if TYPE_CHECKING:
    from typing import Any


@dataclass
class FileToPersist:
    file_name: str
    file_content: str | None = None

    def set_file_content(self, file_content: str) -> None:
        self.file_content = file_content


class ResultBackend(ABC):
    def __init__(self, *args: Any, **kwargs: Any):
        self.logger = logging.getLogger(self.__class__.__name__)

    def write(self, connector: Connector, file_to_persist: FileToPersist) -> None:
        if not file_to_persist.file_content:
            raise ValueError(
                "File content must be set before writing to local directory"
            )
        self._write(connector, file_to_persist)

    @abstractmethod
    def _write(self, connector: Connector, file_to_persist: FileToPersist) -> None:
        raise NotImplementedError("write method must be implemented by subclass")

    @abstractmethod
    def artifact_already_exists(
        self, connector: Connector, file_to_persist: FileToPersist
    ) -> bool:
        raise NotImplementedError(
            "insights_already_exist method must be implemented by subclass"
        )


class LocalDir(ResultBackend):
    def __init__(self, local_directory: Path):
        super().__init__()
        if not local_directory.exists():
            local_directory.mkdir(parents=True)

        self.local_directory = local_directory

    def _write(self, connector: Connector, file_to_persist: FileToPersist) -> None:
        assert file_to_persist.file_content is not None
        connector_result_directory = (
            self.local_directory / connector.technical_name / connector.version
        )
        connector_result_directory.mkdir(parents=True, exist_ok=True)
        file_path = connector_result_directory / file_to_persist.file_name
        with open(file_path, "w") as f:
            f.write(file_to_persist.file_content)
            self.logger.info(f"{file_to_persist.file_name} written to {file_path}")

    def artifact_already_exists(
        self, connector: Connector, file_to_persist: FileToPersist
    ) -> bool:
        connector_result_directory = (
            self.local_directory / connector.technical_name / connector.version
        )
        return (connector_result_directory / file_to_persist.file_name).exists()


class GCSBucket(ResultBackend):
    DEFAULT_GCP_PROJECT = "prod-ab-cloud-proj"

    def __init__(
        self, bucket_name: str, key_prefix: str, gcp_project: str = DEFAULT_GCP_PROJECT
    ):
        super().__init__()
        self.bucket_name = bucket_name
        self.key_prefix = key_prefix
        self.storage_client = storage.Client(project=gcp_project)
        self.bucket = self.storage_client.bucket(self.bucket_name)

    def _write(self, connector: Connector, file_to_persist: FileToPersist) -> None:
        assert file_to_persist.file_content is not None
        version_blob_prefix = (
            f"{self.key_prefix}/{connector.technical_name}/{connector.version}"
        )
        latest_blob_prefix = f"{self.key_prefix}/{connector.technical_name}/latest"
        for blob_prefix in [version_blob_prefix, latest_blob_prefix]:
            blob = self.bucket.blob(f"{blob_prefix}/{file_to_persist.file_name}")
            blob.upload_from_string(file_to_persist.file_content)
            self.logger.info(
                f"{file_to_persist.file_name} written to {blob.public_url}"
            )

    def artifact_already_exists(
        self, connector: Connector, file_to_persist: FileToPersist
    ) -> bool:
        blob_prefix = (
            f"{self.key_prefix}/{connector.technical_name}/{connector.version}"
        )
        return self.bucket.blob(f"{blob_prefix}/{file_to_persist.file_name}").exists()



#!/usr/bin/env python3

# Copyright (c) 2024 Airbyte, Inc., all rights reserved.
"""Generate docs for all public modules in airbyte-ops-mcp and save them to docs/generated.

Usage:
    uv run python docs/generate.py

Or with Poe-the-Poet:
    poe docs-generate
    poe docs-preview

"""

from __future__ import annotations

import pathlib
import shutil

import pdoc


def run() -> None:
    """Generate docs for all public modules in airbyte-ops-mcp and save them to docs/generated."""
    public_modules = ["airbyte_ops_mcp"]

    # recursively delete the docs/generated folder if it exists
    if pathlib.Path("docs/generated").exists():
        shutil.rmtree("docs/generated")

    pdoc.render.configure(
        template_directory=pathlib.Path("docs/templates"),
        show_source=True,
        search=True,
        logo="https://docs.airbyte.com/img/logo-dark.png",
        favicon="https://docs.airbyte.com/img/favicon.png",
        mermaid=True,
        docformat="google",
    )
    pdoc.pdoc(
        *public_modules,
        output_directory=pathlib.Path("docs/generated"),
    )


if __name__ == "__main__":
    run()



#
# Copyright (c) 2023 Airbyte, Inc., all rights reserved.
#

"""This module groups steps made to run tests agnostic to a connector language."""

import datetime
import json
import os
import time
from abc import ABC, abstractmethod
from enum import Enum
from functools import cached_property
from pathlib import Path
from textwrap import dedent
from typing import Any, Dict, List, Optional, Set

import dagger
import requests  # type: ignore
import semver
import yaml  # type: ignore
from consts import INTERNAL_TOOL_PATHS, CIContext
from dagger import Container, Directory
from pipelines import hacks, main_logger
from pipelines.airbyte_ci.connectors.consts import CONNECTOR_TEST_STEP_ID
from pipelines.airbyte_ci.connectors.context import ConnectorContext
from pipelines.airbyte_ci.steps.docker import SimpleDockerStep
from pipelines.dagger.actions import secrets
from pipelines.dagger.actions.python.poetry import with_poetry
from pipelines.helpers.github import AIRBYTE_GITHUBUSERCONTENT_URL_PREFIX
from pipelines.helpers.utils import METADATA_FILE_NAME, get_exec_result
from pipelines.models.artifacts import Artifact
from pipelines.models.secrets import Secret
from pipelines.models.steps import STEP_PARAMS, MountPath, Step, StepResult, StepStatus

# This slugify lib has to be consistent with the slugify lib used in live_tests
# live_test can't resolve the passed connector container otherwise.
from slugify import slugify  # type: ignore

GITHUB_URL_PREFIX_FOR_CONNECTORS = (
    f"{AIRBYTE_GITHUBUSERCONTENT_URL_PREFIX}/master/airbyte-integrations/connectors"
)


class VersionCheck(Step, ABC):
    """A step to validate the connector version was bumped if files were modified"""

    context: ConnectorContext

    @property
    def should_run(self) -> bool:
        return True

    @property
    def github_master_metadata_url(self) -> str:
        return f"{GITHUB_URL_PREFIX_FOR_CONNECTORS}/{self.context.connector.technical_name}/{METADATA_FILE_NAME}"

    @cached_property
    def master_metadata(self) -> Optional[dict]:
        response = requests.get(self.github_master_metadata_url)

        # New connectors will not have a metadata file in master
        if not response.ok:
            return None
        return yaml.safe_load(response.text)

    @property
    def master_connector_version(self) -> semver.Version:
        metadata = self.master_metadata
        if not metadata:
            return semver.Version.parse("0.0.0")

        return semver.Version.parse(str(metadata["data"]["dockerImageTag"]))

    @property
    def current_connector_version(self) -> semver.Version:
        return semver.Version.parse(str(self.context.metadata["dockerImageTag"]))

    @property
    def success_result(self) -> StepResult:
        return StepResult(step=self, status=StepStatus.SUCCESS)

    def _get_failure_result(self, failure_message: str) -> StepResult:
        return StepResult(step=self, status=StepStatus.FAILURE, stderr=failure_message)

    @abstractmethod
    def validate(self) -> StepResult:
        raise NotImplementedError()

    async def _run(self) -> StepResult:
        if not self.should_run:
            return StepResult(
                step=self,
                status=StepStatus.SKIPPED,
                stdout="No modified files required a version bump.",
            )
        if self.context.ci_context == CIContext.MASTER:
            return StepResult(
                step=self,
                status=StepStatus.SKIPPED,
                stdout="Version check are not running in master context.",
            )
        try:
            return self.validate()
        except (requests.HTTPError, ValueError, TypeError) as e:
            return StepResult(step=self, status=StepStatus.FAILURE, stderr=str(e))


class VersionIncrementCheck(VersionCheck):
    context: ConnectorContext
    title = "Connector version increment check"

    BYPASS_CHECK_FOR = [
        METADATA_FILE_NAME,
        "acceptance-test-config.yml",
        "README.md",
        "bootstrap.md",
        ".dockerignore",
        "unit_tests",
        "integration_tests",
        "src/test",
        "src/test-integration",
        "src/test-performance",
        "build.gradle",
        "erd",
        "build_customization.py",
    ]

    @property
    def should_run(self) -> bool:
        # Skip if connector opts out of version checks
        if (
            self.context.metadata
            and self.context.metadata.get("ab_internal", {}).get(
                "requireVersionIncrementsInPullRequests"
            )
            is False
        ):
            return False

        for filename in self.context.modified_files:
            relative_path = str(filename).replace(
                str(self.context.connector.code_directory) + "/", ""
            )
            if not any(
                [
                    relative_path.startswith(to_bypass)
                    for to_bypass in self.BYPASS_CHECK_FOR
                ]
            ):
                return True
        return False

    def is_version_not_incremented(self) -> bool:
        return self.master_connector_version >= self.current_connector_version

    def get_failure_message_for_no_increment(self) -> str:
        return (
            f"The dockerImageTag in {METADATA_FILE_NAME} was not incremented. "
            f"Master version is {self.master_connector_version}, current version is {self.current_connector_version}"
        )

    def are_both_versions_release_candidates(self) -> bool:
        return bool(
            self.master_connector_version.prerelease
            and self.current_connector_version.prerelease
            and "rc" in self.master_connector_version.prerelease
            and "rc" in self.current_connector_version.prerelease
        )

    def have_same_major_minor_patch(self) -> bool:
        return (
            self.master_connector_version.major == self.current_connector_version.major
            and self.master_connector_version.minor
            == self.current_connector_version.minor
            and self.master_connector_version.patch
            == self.current_connector_version.patch
        )

    def validate(self) -> StepResult:
        if self.is_version_not_incremented():
            return self._get_failure_result(
                f"The dockerImageTag in {METADATA_FILE_NAME} was not incremented. "
                f"Master version is {self.master_connector_version}, current version is {self.current_connector_version}"
            )

        if self.are_both_versions_release_candidates():
            if not self.have_same_major_minor_patch():
                return self._get_failure_result(
                    f"Master and current version are release candidates but they have different major, minor or patch versions. "
                    f"Release candidates should only differ in the prerelease part. Master version is {self.master_connector_version}, "
                    f"current version is {self.current_connector_version}"
                )

        return self.success_result


class QaChecks(SimpleDockerStep):
    """A step to run QA checks for a connectors.
    More details in https://github.com/airbytehq/airbyte/blob/main/airbyte-ci/connectors/connectors_qa/README.md
    """

    def __init__(self, context: ConnectorContext) -> None:
        code_directory = context.connector.code_directory
        documentation_file_path = context.connector.documentation_file_path
        migration_guide_file_path = context.connector.migration_guide_file_path
        icon_path = context.connector.icon_path
        technical_name = context.connector.technical_name

        # When the connector is strict-encrypt, we should run QA checks on the main one as it's the one whose artifacts gets released
        if context.connector.technical_name.endswith("-strict-encrypt"):
            technical_name = technical_name.replace("-strict-encrypt", "")
            code_directory = Path(str(code_directory).replace("-strict-encrypt", ""))
            if documentation_file_path:
                documentation_file_path = Path(
                    str(documentation_file_path).replace("-strict-encrypt", "")
                )
            if migration_guide_file_path:
                migration_guide_file_path = Path(
                    str(migration_guide_file_path).replace("-strict-encrypt", "")
                )
            if icon_path:
                icon_path = Path(str(icon_path).replace("-strict-encrypt", ""))

        super().__init__(
            title=f"Run QA checks for {technical_name}",
            context=context,
            paths_to_mount=[
                MountPath(code_directory),
                # These paths are optional
                # But their absence might make the QA check fail
                MountPath(documentation_file_path, optional=True),
                MountPath(migration_guide_file_path, optional=True),
                MountPath(icon_path, optional=True),
            ],
            internal_tools=[
                MountPath(INTERNAL_TOOL_PATHS.CONNECTORS_QA.value),
            ],
            secret_env_variables={
                "DOCKER_HUB_USERNAME": context.docker_hub_username,
                "DOCKER_HUB_PASSWORD": context.docker_hub_password,
            }
            if context.docker_hub_username and context.docker_hub_password
            else None,
            command=["connectors-qa", "run", f"--name={technical_name}"],
        )


class AcceptanceTests(Step):
    """A step to run acceptance tests for a connector if it has an acceptance test config file."""

    context: ConnectorContext
    title = "Acceptance tests"
    CONTAINER_TEST_INPUT_DIRECTORY = "/test_input"
    CONTAINER_SECRETS_DIRECTORY = "/test_input/secrets"
    REPORT_LOG_PATH = "/tmp/report_log.jsonl"
    skipped_exit_code = 5
    accept_extra_params = True

    @property
    def default_params(self) -> STEP_PARAMS:
        """Default pytest options.

        Returns:
            dict: The default pytest options.
        """
        return (
            super().default_params
            | {
                "-ra": [],  # Show extra test summary info in the report for all but the passed tests
                "--disable-warnings": [],  # Disable warnings in the pytest report
                "--durations": ["3"],  # Show the 3 slowest tests in the report
            }
        )

    @property
    def base_cat_command(self) -> List[str]:
        command = [
            "python",
            "-m",
            "pytest",
            # Write the test report in jsonl format
            f"--report-log={self.REPORT_LOG_PATH}",
            "-p",  # Load the connector_acceptance_test plugin
            "connector_acceptance_test.plugin",
            "--acceptance-test-config",
            self.CONTAINER_TEST_INPUT_DIRECTORY,
        ]

        if self.concurrent_test_run:
            command += [
                "--numprocesses=auto"
            ]  # Using pytest-xdist to run tests in parallel, auto means using all available cores
        return command

    def __init__(
        self,
        context: ConnectorContext,
        secrets: List[Secret],
        concurrent_test_run: Optional[bool] = False,
    ) -> None:
        """Create a step to run acceptance tests for a connector if it has an acceptance test config file.

        Args:
            context (ConnectorContext): The current test context, providing a connector object, a dagger client and a repository directory.
            secrets (List[Secret]): List of secrets to mount to the connector container under test.
            concurrent_test_run (Optional[bool], optional): Whether to run acceptance tests in parallel. Defaults to False.
        """
        super().__init__(context, secrets)
        self.concurrent_test_run = concurrent_test_run

    async def get_cat_command(self, connector_dir: Directory) -> List[str]:
        """
        Connectors can optionally setup or teardown resources before and after the acceptance tests are run.
        This is done via the acceptance.py file in their integration_tests directory.
        We append this module as a plugin the acceptance will use.
        """
        cat_command = self.base_cat_command
        if "integration_tests" in await connector_dir.entries():
            if (
                "acceptance.py"
                in await connector_dir.directory("integration_tests").entries()
            ):
                cat_command += ["-p", "integration_tests.acceptance"]
        return cat_command + self.params_as_cli_options

    async def _run(self, connector_under_test_container: Container) -> StepResult:
        """Run the acceptance test suite on a connector dev image. Build the connector acceptance test image if the tag is :dev.

        Args:
            connector_under_test_container (Container): The container holding the connector under test image.

        Returns:
            StepResult: Failure or success of the acceptances tests with stdout and stderr.
        """

        if not self.context.connector.acceptance_test_config:
            return StepResult(step=self, status=StepStatus.SKIPPED)
        connector_dir = await self.context.get_connector_dir()
        cat_container = await self._build_connector_acceptance_test(
            connector_under_test_container, connector_dir
        )
        cat_command = await self.get_cat_command(connector_dir)
        cat_container = cat_container.with_(hacks.never_fail_exec(cat_command))
        step_result = await self.get_step_result(cat_container)
        secret_dir = cat_container.directory(self.CONTAINER_SECRETS_DIRECTORY)

        if secret_files := await secret_dir.entries():
            for file_path in secret_files:
                if file_path.startswith("updated_configurations"):
                    self.context.updated_secrets_dir = secret_dir
                    break
        return step_result

    def get_cache_buster(self) -> str:
        """
        This bursts the CAT cached results everyday and on new version or image size change.
        It's cool because in case of a partially failing nightly build the connectors that already ran CAT won't re-run CAT.
        We keep the guarantee that a CAT runs everyday.

        Returns:
            str: A string representing the cachebuster value.
        """
        return (
            datetime.datetime.utcnow().strftime("%Y%m%d")
            + self.context.connector.version
        )

    async def _build_connector_acceptance_test(
        self, connector_under_test_container: Container, test_input: Directory
    ) -> Container:
        """Create a container to run connector acceptance tests.

        Args:
            connector_under_test_container (Container): The container holding the connector under test image.
            test_input (Directory): The connector under test directory.
        Returns:
            Container: A container with connector acceptance tests installed.
        """

        if self.context.connector_acceptance_test_image.endswith(":dev"):
            cat_container = (
                self.context.connector_acceptance_test_source_dir.docker_build()
            )
        else:
            cat_container = self.dagger_client.container().from_(
                self.context.connector_acceptance_test_image
            )

        connector_container_id = await connector_under_test_container.id()
        cat_container = (
            cat_container.with_env_variable("RUN_IN_AIRBYTE_CI", "1")
            .with_exec(["mkdir", "/dagger_share"])
            .with_env_variable("CACHEBUSTER", self.get_cache_buster())
            .with_new_file(
                "/tmp/container_id.txt", contents=str(connector_container_id)
            )
            .with_workdir("/test_input")
            .with_mounted_directory("/test_input", test_input)
            .with_(
                await secrets.mounted_connector_secrets(
                    self.context, self.CONTAINER_SECRETS_DIRECTORY, self.secrets
                )
            )
        )
        if "_EXPERIMENTAL_DAGGER_RUNNER_HOST" in os.environ:
            self.context.logger.info(
                "Using experimental dagger runner host to run CAT with dagger-in-dagger"
            )
            cat_container = cat_container.with_env_variable(
                "_EXPERIMENTAL_DAGGER_RUNNER_HOST",
                "unix:///var/run/buildkit/buildkitd.sock",
            ).with_unix_socket(
                "/var/run/buildkit/buildkitd.sock",
                self.context.dagger_client.host().unix_socket(
                    "/var/run/buildkit/buildkitd.sock"
                ),
            )

        return cat_container.with_unix_socket(
            "/var/run/docker.sock",
            self.context.dagger_client.host().unix_socket("/var/run/docker.sock"),
        )

    def get_is_hard_failure(self) -> bool:
        """When a connector is not certified or the CI context is master, we consider the acceptance tests as hard failures:
        The overall status of the pipeline will be FAILURE if the acceptance tests fail.
        For marketplace connectors we defer to the IncrementalAcceptanceTests step to determine if the acceptance tests are hard failures:
        If a new test is failing compared to the released version of the connector.

        Returns:
            bool: Whether a failure of acceptance tests should be considered a hard failures.
        """
        return (
            self.context.connector.metadata.get("supportLevel") == "certified"
            or self.context.ci_context == CIContext.MASTER
        )

    async def get_step_result(self, container: Container) -> StepResult:
        """Retrieve stdout, stderr and exit code from the executed CAT container.
        Pull the report logs from the container and create an Artifact object from it.
        Build and return a step result object from these objects.

        Args:
            container (Container): The CAT container to get the results from.

        Returns:
            StepResult: The step result object.
        """
        exit_code, stdout, stderr = await get_exec_result(container)
        report_log_artifact = Artifact(
            name="cat_report_log.jsonl",
            content_type="text/jsonl",
            content=container.file(self.REPORT_LOG_PATH),
            to_upload=True,
        )
        status = self.get_step_status_from_exit_code(exit_code)

        is_hard_failure = status is StepStatus.FAILURE and self.get_is_hard_failure()

        return StepResult(
            step=self,
            status=self.get_step_status_from_exit_code(exit_code),
            stderr=stderr,
            stdout=stdout,
            output={"report_log": report_log_artifact},
            artifacts=[report_log_artifact],
            consider_in_overall_status=status is StepStatus.SUCCESS or is_hard_failure,
        )


class IncrementalAcceptanceTests(Step):
    """This step runs the acceptance tests on the released image of the connector and compares the results with the current acceptance tests report log.
    It fails if there are new failing tests in the current acceptance tests report log.
    """

    title = "Incremental Acceptance Tests"
    context: ConnectorContext

    async def get_failed_pytest_node_ids(
        self, current_acceptance_tests_report_log: Artifact
    ) -> Set[str]:
        """Parse the report log of the acceptance tests and return the pytest node ids of the failed tests.

        Args:
            current_acceptance_tests_report_log (Artifact): The report log of the acceptance tests.

        Returns:
            List[str]: The pytest node ids of the failed tests.
        """
        current_report_lines = (
            await current_acceptance_tests_report_log.content.contents()
        ).splitlines()
        failed_nodes = set()
        for line in current_report_lines:
            single_test_report = json.loads(line)
            if (
                "nodeid" not in single_test_report
                or "outcome" not in single_test_report
            ):
                continue
            if single_test_report["outcome"] == "failed":
                failed_nodes.add(single_test_report["nodeid"])
        return failed_nodes

    def _get_master_metadata(self) -> Dict[str, Any]:
        metadata_response = requests.get(
            f"{GITHUB_URL_PREFIX_FOR_CONNECTORS}/{self.context.connector.technical_name}/metadata.yaml"
        )
        if not metadata_response.ok:
            raise FileNotFoundError(
                f"Could not fetch metadata file for {self.context.connector.technical_name} on master."
            )
        return yaml.safe_load(metadata_response.text)

    async def get_result_log_on_master(self, master_metadata: dict) -> Artifact:
        """Runs acceptance test on the released image of the connector and returns the report log.
        The released image version is fetched from the master metadata file of the connector.
        We're not using the online connector registry here as some connectors might not be released to OSS nor Airbyte Cloud.
        Thanks to Dagger caching subsequent runs of this step will be cached if the released image did not change.

        Returns:
            Artifact: The report log of the acceptance tests run on the released image.
        """
        master_docker_image_tag = master_metadata["data"]["dockerImageTag"]
        released_image = (
            f"{master_metadata['data']['dockerRepository']}:{master_docker_image_tag}"
        )
        released_container = self.dagger_client.container().from_(released_image)
        self.logger.info(
            f"Running acceptance tests on released image: {released_image}"
        )
        acceptance_tests_results_on_master = await AcceptanceTests(
            self.context, self.secrets
        ).run(released_container)
        return acceptance_tests_results_on_master.output["report_log"]

    async def _run(self, current_acceptance_tests_result: StepResult) -> StepResult:
        """Compare the acceptance tests report log of the current image with the one of the released image.
        Fails if there are new failing tests in the current acceptance tests report log.
        """

        if current_acceptance_tests_result.consider_in_overall_status:
            return StepResult(
                step=self,
                status=StepStatus.SKIPPED,
                stdout="Skipping because the current acceptance tests are hard failures.",
            )

        current_acceptance_tests_report_log = current_acceptance_tests_result.output[
            "report_log"
        ]
        current_failing_nodes = await self.get_failed_pytest_node_ids(
            current_acceptance_tests_report_log
        )
        if not current_failing_nodes:
            return StepResult(
                step=self,
                status=StepStatus.SKIPPED,
                stdout="No failing acceptance tests were detected on the current version.",
            )
        try:
            master_metadata = self._get_master_metadata()
        except FileNotFoundError as exc:
            return StepResult(
                step=self,
                status=StepStatus.SKIPPED,
                stdout="The connector does not have a metadata file on master. Skipping incremental acceptance tests.",
                exc_info=exc,
            )

        master_result_logs = await self.get_result_log_on_master(master_metadata)

        master_failings = await self.get_failed_pytest_node_ids(master_result_logs)
        new_failing_nodes = current_failing_nodes - master_failings
        if not new_failing_nodes:
            return StepResult(
                step=self,
                status=StepStatus.SUCCESS,
                stdout=dedent(
                    f"""
                No new failing acceptance tests were detected.
                Acceptance tests are still failing with {len(current_failing_nodes)} failing tests but the AcceptanceTests step is not a hard failure for this connector.
                Please checkout the original acceptance tests failures and assess how critical they are.
                """
                ),
            )
        else:
            return StepResult(
                step=self,
                status=StepStatus.FAILURE,
                stdout=f"{len(new_failing_nodes)} new failing acceptance tests detected:\n-"
                + "\n-".join(current_failing_nodes)
                + "\nPlease fix the new failing tests before merging this PR."
                + f"\nPlease also check the original {len(current_failing_nodes)} acceptance tests failures and assess how critical they are.",
            )


class LiveTestSuite(Enum):
    ALL = "live"
    REGRESSION = "regression"
    VALIDATION = "validation"


class LiveTests(Step):
    """A step to run live tests for a connector."""

    context: ConnectorContext
    skipped_exit_code = 5
    accept_extra_params = True
    local_tests_artifacts_dir = Path("/tmp/live_tests_artifacts")
    working_directory = "/app"
    github_user = "octavia-squidington-iii"
    platform_repo_url = "airbytehq/airbyte-platform-internal"
    test_suite_to_dir = {
        LiveTestSuite.ALL: "src/live_tests",
        LiveTestSuite.REGRESSION: "src/live_tests/regression_tests",
        LiveTestSuite.VALIDATION: "src/live_tests/validation_tests",
    }

    @property
    def default_params(self) -> STEP_PARAMS:
        """Default pytest options.

        Returns:
            dict: The default pytest options.
        """
        return (
            super().default_params
            | {
                "-ra": [],  # Show extra test summary info in the report for all but the passed tests
                "--disable-warnings": [],  # Disable warnings in the pytest report
                "--durations": ["3"],  # Show the 3 slowest tests in the report
            }
        )

    @property
    def title(self) -> str:
        return f"Connector {self.test_suite.title()} Tests"

    def _test_command(self) -> List[str]:
        """
        The command used to run the tests
        """
        base_command = [
            "poetry",
            "run",
            "pytest",
            self.test_dir,
            "--connector-image",
            self.connector_image,
        ]
        return base_command + self._get_command_options()

    def _get_command_options(self) -> List[str]:
        command_options = []
        if self.connection_id:
            command_options += ["--connection-id", self.connection_id]
        if self.control_version:
            command_options += ["--control-version", self.control_version]
        if self.target_version:
            command_options += ["--target-version", self.target_version]
        if self.pr_url:
            command_options += ["--pr-url", self.pr_url]
        if self.run_id:
            command_options += ["--run-id", self.run_id]
        if self.should_read_with_state:
            command_options += ["--should-read-with-state=1"]
        if self.disable_proxy:
            command_options += ["--disable-proxy=1"]
        if self.test_evaluation_mode:
            command_options += ["--test-evaluation-mode", self.test_evaluation_mode]
        if self.selected_streams:
            command_options += ["--stream", self.selected_streams]
        command_options += ["--connection-subset", self.connection_subset]
        return command_options

    def _run_command_with_proxy(self, command: str) -> List[str]:
        """
        This command:

        1. Starts a Google Cloud SQL proxy running on localhost, which is used by the connection-retriever to connect to postgres.
           This is required for secure access to our internal tools.
        2. Gets the PID of the proxy so it can be killed once done.
        3. Runs the command that was passed in as input.
        4. Kills the proxy, and waits for it to exit.
        5. Exits with the command's exit code.
        We need to explicitly kill the proxy in order to allow the GitHub Action to exit.
        An alternative that we can consider is to run the proxy as a separate service.

        (See https://docs.dagger.io/manuals/developer/python/328492/services/ and https://cloud.google.com/sql/docs/postgres/sql-proxy#cloud-sql-auth-proxy-docker-image)
        """
        run_proxy = "./cloud-sql-proxy prod-ab-cloud-proj:us-west3:prod-pgsql-replica --credentials-file /tmp/credentials.json"
        run_pytest_with_proxy = dedent(
            f"""
        {run_proxy} &
        proxy_pid=$!
        {command}
        pytest_exit=$?
        kill $proxy_pid
        wait $proxy_pid
        exit $pytest_exit
        """
        )
        return ["bash", "-c", f"'{run_pytest_with_proxy}'"]

    def __init__(self, context: ConnectorContext) -> None:
        """Create a step to run live tests for a connector.

        Args:
            context (ConnectorContext): The current test context, providing a connector object, a dagger client and a repository directory.
        """
        super().__init__(context)
        self.connector_image = context.docker_image.split(":")[0]
        options = self.context.run_step_options.step_params.get(
            CONNECTOR_TEST_STEP_ID.CONNECTOR_LIVE_TESTS, {}
        )

        self.test_suite = self.context.run_step_options.get_item_or_default(
            options, "test-suite", LiveTestSuite.REGRESSION.value
        )
        self.connection_id = self._get_connection_id(options)
        self.pr_url = self._get_pr_url(options)

        self.test_dir = self.test_suite_to_dir[LiveTestSuite(self.test_suite)]
        self.control_version = self.context.run_step_options.get_item_or_default(
            options, "control-version", None
        )
        self.target_version = self.context.run_step_options.get_item_or_default(
            options, "target-version", "dev"
        )
        self.should_read_with_state = "should-read-with-state" in options
        self.disable_proxy = "disable-proxy" in options
        self.selected_streams = self.context.run_step_options.get_item_or_default(
            options, "selected-streams", None
        )
        self.test_evaluation_mode = (
            "strict"
            if self.context.connector.metadata.get("supportLevel") == "certified"
            else "diagnostic"
        )
        self.connection_subset = self.context.run_step_options.get_item_or_default(
            options, "connection-subset", "sandboxes"
        )
        self.run_id = os.getenv("GITHUB_RUN_ID") or str(int(time.time()))

    def _get_connection_id(self, options: Dict[str, List[Any]]) -> Optional[str]:
        if self.context.is_pr:
            connection_id = self._get_connection_from_test_connections()
            self.logger.info(
                f"Context is {self.context.ci_context}; got connection_id={connection_id} from metadata.yaml liveTests testConnections."
            )
        else:
            connection_id = self.context.run_step_options.get_item_or_default(
                options, "connection-id", None
            )
            self.logger.info(
                f"Context is {self.context.ci_context}; got connection_id={connection_id} from input options."
            )
        return connection_id

    def _get_pr_url(self, options: Dict[str, List[Any]]) -> Optional[str]:
        if self.context.is_pr:
            pull_request = (
                self.context.pull_request.url if self.context.pull_request else None
            )
            self.logger.info(
                f"Context is {self.context.ci_context}; got pull_request={pull_request} from context."
            )
        else:
            pull_request = self.context.run_step_options.get_item_or_default(
                options, "pr-url", None
            )
            self.logger.info(
                f"Context is {self.context.ci_context}; got pull_request={pull_request} from input options."
            )
        return pull_request

    def _validate_job_can_run(self) -> None:
        connector_type = self.context.connector.metadata.get("connectorType")
        connector_subtype = self.context.connector.metadata.get("connectorSubtype")
        assert connector_type == "source", (
            f"Live tests can only run against source connectors, got `connectorType={connector_type}`."
        )
        if connector_subtype == "database":
            assert self.connection_subset == "sandboxes", (
                f"Live tests for database sources may only be run against sandbox connections, got `connection_subset={self.connection_subset}`."
            )

        assert self.connection_id, "`connection-id` is required to run live tests."
        assert self.pr_url, "`pr_url` is required to run live tests."

        if self.context.is_pr:
            connection_id_is_valid = False
            for test_suite in self.context.connector.metadata.get(
                "connectorTestSuitesOptions", []
            ):
                if test_suite["suite"] == "liveTests":
                    assert self.connection_id in [
                        option["id"] for option in test_suite.get("testConnections", [])
                    ], (
                        f"Connection ID {self.connection_id} was not in the list of valid test connections."
                    )
                    connection_id_is_valid = True
                    break
            assert connection_id_is_valid, (
                f"Connection ID {self.connection_id} is not a valid sandbox connection ID."
            )

    def _get_connection_from_test_connections(self) -> Optional[str]:
        for test_suite in self.context.connector.metadata.get(
            "connectorTestSuitesOptions", []
        ):
            if test_suite["suite"] == "liveTests":
                for option in test_suite.get("testConnections", []):
                    connection_id = option["id"]
                    connection_name = option["name"]
                    self.logger.info(
                        f"Using connection name={connection_name}; id={connection_id}"
                    )
                    return connection_id
        return None

    async def _run(self, connector_under_test_container: Container) -> StepResult:
        """Run the regression test suite.

        Args:
            connector_under_test (Container): The container holding the target connector test image.

        Returns:
            StepResult: Failure or success of the regression tests with stdout and stderr.
        """
        try:
            self._validate_job_can_run()
        except AssertionError as exc:
            self.logger.info(
                f"Skipping live tests for {self.context.connector.technical_name} due to validation error {exc!s}."
            )
            return StepResult(
                step=self,
                status=StepStatus.SKIPPED,
                exc_info=exc,
            )

        container = await self._build_test_container(
            await connector_under_test_container.id()
        )
        command = self._run_command_with_proxy(" ".join(self._test_command()))
        main_logger.info(f"Running command {command}")
        container = container.with_(hacks.never_fail_exec(command))
        tests_artifacts_dir = str(self.local_tests_artifacts_dir)
        path_to_report = f"{tests_artifacts_dir}/session_{self.run_id}/report.html"

        exit_code, stdout, stderr = await get_exec_result(container)

        try:
            if (
                f"session_{self.run_id}"
                not in await container.directory(f"{tests_artifacts_dir}").entries()
                or "report.html"
                not in await container.directory(
                    f"{tests_artifacts_dir}/session_{self.run_id}"
                ).entries()
            ):
                main_logger.exception(
                    "The report file was not generated, an unhandled error likely happened during regression test execution, please check the step stderr and stdout for more details"
                )
                regression_test_report = None
            else:
                await container.file(path_to_report).export(path_to_report)
                with open(path_to_report) as fp:
                    regression_test_report = fp.read()
        except dagger.QueryError as exc:
            regression_test_report = None
            main_logger.exception(
                "The test artifacts directory was not generated, an unhandled error likely happened during setup, please check the step stderr and stdout for more details",
                exc_info=exc,
            )

        return StepResult(
            step=self,
            status=self.get_step_status_from_exit_code(exit_code),
            stderr=stderr,
            stdout=stdout,
            output=container,
            report=regression_test_report,
            consider_in_overall_status=False if self.context.is_pr else True,
        )

    async def _build_test_container(self, target_container_id: str) -> Container:
        """Create a container to run regression tests."""
        container = with_poetry(self.context)
        container_requirements = [
            "apt-get",
            "install",
            "-y",
            "git",
            "curl",
            "docker.io",
        ]
        if not self.context.is_ci:
            # Outside of CI we use ssh to get the connection-retriever package from airbyte-platform-internal
            container_requirements += ["openssh-client"]
        container = (
            container.with_exec(["apt-get", "update"], use_entrypoint=True)
            .with_exec(container_requirements)
            .with_exec(
                ["bash", "-c", "curl https://sdk.cloud.google.com | bash"],
                use_entrypoint=True,
            )
            .with_env_variable("PATH", "/root/google-cloud-sdk/bin:$PATH", expand=True)
            .with_mounted_directory("/app", self.context.live_tests_dir)
            .with_workdir("/app")
            # Enable dagger-in-dagger
            .with_unix_socket(
                "/var/run/docker.sock",
                self.dagger_client.host().unix_socket("/var/run/docker.sock"),
            )
            .with_env_variable("RUN_IN_AIRBYTE_CI", "1")
            .with_file(
                "/tmp/record_obfuscator.py",
                self.context.get_repo_dir(
                    "tools/bin", include=["record_obfuscator.py"]
                ).file("record_obfuscator.py"),
            )
            # The connector being tested is already built and is stored in a location accessible to an inner dagger kicked off by
            # regression tests. The connector can be found if you know the container ID, so we write the container ID to a file and put
            # it in the regression test container. This way regression tests will use the already-built connector instead of trying to
            # build their own.
            .with_new_file(
                f"/tmp/{slugify(self.connector_image + ':' + self.target_version)}_container_id.txt",
                contents=str(target_container_id),
            )
        )

        if self.context.is_ci:
            container = (
                container.with_exec(
                    [
                        "sed",
                        "-i",
                        "-E",
                        rf"s,git@github\.com:{self.platform_repo_url},https://github.com/{self.platform_repo_url}.git,",
                        "pyproject.toml",
                    ],
                    use_entrypoint=True,
                )
                .with_exec(
                    [
                        "poetry",
                        "source",
                        "add",
                        "--priority=supplemental",
                        "airbyte-platform-internal-source",
                        "https://github.com/airbytehq/airbyte-platform-internal.git",
                    ],
                    use_entrypoint=True,
                )
                .with_secret_variable(
                    "CI_GITHUB_ACCESS_TOKEN",
                    self.context.dagger_client.set_secret(
                        "CI_GITHUB_ACCESS_TOKEN",
                        self.context.ci_github_access_token.value
                        if self.context.ci_github_access_token
                        else "",
                    ),
                )
                .with_exec(
                    [
                        "/bin/sh",
                        "-c",
                        f"poetry config http-basic.airbyte-platform-internal-source {self.github_user} $CI_GITHUB_ACCESS_TOKEN",
                    ],
                    use_entrypoint=True,
                )
                # Add GCP credentials from the environment and point google to their location (also required for connection-retriever)
                .with_new_file(
                    "/tmp/credentials.json",
                    contents=os.getenv("GCP_PROD_DB_ACCESS_CREDENTIALS", ""),
                )
                .with_env_variable(
                    "GOOGLE_APPLICATION_CREDENTIALS", "/tmp/credentials.json"
                )
                .with_exec(
                    [
                        "curl",
                        "-o",
                        "cloud-sql-proxy",
                        "https://storage.googleapis.com/cloud-sql-connectors/cloud-sql-proxy/v2.11.0/cloud-sql-proxy.linux.amd64",
                    ],
                    use_entrypoint=True,
                )
                .with_exec(["chmod", "+x", "cloud-sql-proxy"], use_entrypoint=True)
                .with_env_variable("CI", "1")
            )

        else:
            container = (
                container.with_mounted_file(
                    "/root/.ssh/id_rsa",
                    self.dagger_client.host().file(
                        str(Path("~/.ssh/id_rsa").expanduser())
                    ),
                )
                .with_mounted_file(
                    "/root/.ssh/known_hosts",
                    self.dagger_client.host().file(
                        str(Path("~/.ssh/known_hosts").expanduser())
                    ),
                )
                .with_mounted_file(
                    "/root/.config/gcloud/application_default_credentials.json",
                    self.dagger_client.host().file(
                        str(
                            Path(
                                "~/.config/gcloud/application_default_credentials.json"
                            ).expanduser()
                        )
                    ),
                )
            )

        container = container.with_exec(
            ["poetry", "lock"], use_entrypoint=True
        ).with_exec(["poetry", "install"], use_entrypoint=True)
        return container



# Copyright (c) 2025 Airbyte, Inc., all rights reserved.
"""GCP Cloud Logging utilities for fetching error details by error ID."""

from airbyte_ops_mcp.gcp_logs.error_lookup import (
    GCPLogEntry,
    GCPLogPayload,
    GCPLogSearchResult,
    GCPSeverity,
    fetch_error_logs,
)

__all__ = [
    "GCPLogEntry",
    "GCPLogPayload",
    "GCPLogSearchResult",
    "GCPSeverity",
    "fetch_error_logs",
]



# Copyright (c) 2025 Airbyte, Inc., all rights reserved.
"""Registry operations for Airbyte connectors.

This package provides functionality for:
- Reading connector metadata from the GCS registry
- Listing connectors and versions in the registry
- Publishing connector metadata to GCS
- Promoting and rolling back release candidates
"""

from __future__ import annotations

from airbyte_ops_mcp.registry._constants import (
    DEFAULT_METADATA_SERVICE_BUCKET_NAME,
    DEV_METADATA_SERVICE_BUCKET_NAME,
    LATEST_GCS_FOLDER_NAME,
    METADATA_FILE_NAME,
    METADATA_FOLDER,
    PROD_METADATA_SERVICE_BUCKET_NAME,
    RELEASE_CANDIDATE_GCS_FOLDER_NAME,
    SONAR_DEV_BUCKET_NAME,
    SONAR_PROD_BUCKET_NAME,
)
from airbyte_ops_mcp.registry._enums import (
    ConnectorLanguage,
    ConnectorType,
    SupportLevel,
)
from airbyte_ops_mcp.registry.audit import (
    AuditResult,
    UnpublishedConnector,
    find_unpublished_connectors,
)
from airbyte_ops_mcp.registry.compile import (
    CompileResult,
    PurgeLatestResult,
    compile_registry,
    purge_latest_dirs,
)
from airbyte_ops_mcp.registry.generate import (
    GenerateResult,
    generate_version_artifacts,
)
from airbyte_ops_mcp.registry.models import (
    ConnectorListResult,
    ConnectorMetadata,
    ConnectorPublishResult,
    MetadataPublishResult,
    RegistryEntryResult,
    VersionListResult,
)
from airbyte_ops_mcp.registry.operations import (
    get_registry_entry,
    get_registry_spec,
    list_connector_versions,
    list_registry_connectors,
    list_registry_connectors_filtered,
)
from airbyte_ops_mcp.registry.publish import (
    CONNECTOR_PATH_PREFIX,
    create_progressive_rollout_blob,
    delete_progressive_rollout_blob,
    get_connector_metadata,
    get_gcs_publish_path,
    is_valid_for_progressive_rollout,
    publish_connector_metadata,
    strip_rc_suffix,
)
from airbyte_ops_mcp.registry.publish_artifacts import (
    PublishArtifactsResult,
    publish_version_artifacts,
)
from airbyte_ops_mcp.registry.rebuild import (
    OutputMode,
    RebuildResult,
    rebuild_registry,
)
from airbyte_ops_mcp.registry.registry_store_base import (
    Registry,
    get_registry,
)
from airbyte_ops_mcp.registry.store import (
    REGISTRY_STORE_ENV_VAR,
    RegistryStore,
    StoreType,
    resolve_registry_store,
)
from airbyte_ops_mcp.registry.validate import (
    ValidateOptions,
    ValidationResult,
    validate_metadata,
)
from airbyte_ops_mcp.registry.yank import (
    YANK_FILE_NAME,
    YankResult,
    unyank_connector_version,
    yank_connector_version,
)

__all__ = [
    "CONNECTOR_PATH_PREFIX",
    "DEFAULT_METADATA_SERVICE_BUCKET_NAME",
    "DEV_METADATA_SERVICE_BUCKET_NAME",
    "LATEST_GCS_FOLDER_NAME",
    "METADATA_FILE_NAME",
    "METADATA_FOLDER",
    "PROD_METADATA_SERVICE_BUCKET_NAME",
    "REGISTRY_STORE_ENV_VAR",
    "RELEASE_CANDIDATE_GCS_FOLDER_NAME",
    "SONAR_DEV_BUCKET_NAME",
    "SONAR_PROD_BUCKET_NAME",
    "YANK_FILE_NAME",
    "AuditResult",
    "CompileResult",
    "ConnectorLanguage",
    "ConnectorListResult",
    "ConnectorMetadata",
    "ConnectorPublishResult",
    "ConnectorType",
    "GenerateResult",
    "MetadataPublishResult",
    "OutputMode",
    "PublishArtifactsResult",
    "PurgeLatestResult",
    "RebuildResult",
    "Registry",
    "RegistryEntryResult",
    "RegistryStore",
    "StoreType",
    "SupportLevel",
    "UnpublishedConnector",
    "ValidateOptions",
    "ValidationResult",
    "VersionListResult",
    "YankResult",
    "compile_registry",
    "create_progressive_rollout_blob",
    "delete_progressive_rollout_blob",
    "find_unpublished_connectors",
    "generate_version_artifacts",
    "get_connector_metadata",
    "get_gcs_publish_path",
    "get_registry",
    "get_registry_entry",
    "get_registry_spec",
    "is_valid_for_progressive_rollout",
    "list_connector_versions",
    "list_registry_connectors",
    "list_registry_connectors_filtered",
    "publish_connector_metadata",
    "publish_version_artifacts",
    "purge_latest_dirs",
    "rebuild_registry",
    "resolve_registry_store",
    "strip_rc_suffix",
    "unyank_connector_version",
    "validate_metadata",
    "yank_connector_version",
]
